/* ===============
 * JFreeChart Demo
 * ===============
 *
 * Modified by:		Nathan Conklin (nathan@conklinfamily.net)
 * Project Info:	http://www.object-refinery.com/jfreechart/index.html
 * Project Lead:	David Gilbert (david.gilbert@object-refinery.com);
 *
 */

// Similar to com.jrefiner.chart.demo.PieChartDemo1

package com;

import java.awt.Color;
import java.util.*;
import java.applet.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.sql.*;

import snap.*;

import com.jrefinery.data.DefaultPieDataset;
import com.jrefinery.ui.ApplicationFrame;
import com.jrefinery.chart.*;
import com.jrefinery.chart.event.*;
import com.jrefinery.chart.entity.*;

/**
 * JFreeChart's PieChart wrapped as a Snapplet
 */
public class PieChartSnapplet extends Snapplet 
implements Snapable, ActionListener, ChartMouseListener
{

private JPanel allPanel = new JPanel(new BorderLayout());
private JPanel piePanel = new JPanel(new BorderLayout());
private JPanel menuPanel = new JPanel(new BorderLayout());
private JComboBox menu = new JComboBox();

private ResultSetTableModel rsModel = null;
private int primaryKeyColumnIndex = 0;
private int selectedColumnIndex = 0;

private static final String SELECT = "Select (fire only)";
private static final String MOUSEOVER = "MouseOver (fire only)";

private Object lastMouseOverKey = null;

/** Returns an Enumeration of supported SnapEvents */
public Enumeration getSupportedActions()
{
	Vector v = new Vector();
	v.add(SELECT);
	v.add(MOUSEOVER);
	return v.elements();
}

/** Implementation of Snapable's load() method */
public void load(ResultSet rs, String primaryKeyColumnName)
{
	rsModel = new ResultSetTableModel(rs);
	for (int i = 0; i < rsModel.getColumnCount(); i++ )
	{
		if (rsModel.getColumnName(i).equals(primaryKeyColumnName))
			primaryKeyColumnIndex = i;
	}

	if (selectedColumnIndex > rsModel.getColumnCount())
		selectedColumnIndex = 0;

	buildPopUpMenu();
	updatePieChart();
}

private void updatePieChart()
{
	DefaultPieDataset data = new DefaultPieDataset();

	for (int i = 0; i < rsModel.getRowCount(); i++)
	{
		Object key = rsModel.getValueAt(i, primaryKeyColumnIndex);
		String stringValue =
			rsModel.getValueAt(i, selectedColumnIndex).toString();
		Double value = new Double(getValue(stringValue));
		data.setValue(key, value);
	}

	// create the chart...
	JFreeChart chart = ChartFactory.createPieChart(
		"Pie Chart (JFreeChart)",		// chart title
		data,				// data
		false				// include legend
		);

	// set the background color for the chart...
	chart.setBackgroundPaint(Color.white);

	// add the chart to a panel...
	ChartPanel chartPanel = new ChartPanel(chart);
	chartPanel.addChartMouseListener(this);

	piePanel.removeAll();
	piePanel.add(chartPanel);

	allPanel.revalidate();
	allPanel.repaint();

	this.getContentPane().invalidate();
	this.getContentPane().repaint();

	this.repaint();
}

private void buildPopUpMenu()
{
	menuPanel.remove(menu);

	menu = new JComboBox();                

	for (int i = 0; i < rsModel.getColumnCount(); i++ )
	{
		menu.addItem(rsModel.getColumnName(i));
	}
	menu.setSelectedIndex(selectedColumnIndex);
	menu.addActionListener(this);

	menuPanel.removeAll();
	menuPanel.add(menu);

	allPanel.add(menuPanel, BorderLayout.SOUTH);

	menuPanel.revalidate();
	menuPanel.repaint();
}

/** The column to be charted has changed. */
public void actionPerformed(ActionEvent e)
{
	if (e.getSource() == menu)
	{
		String columnChosen = menu.getSelectedItem().toString();

		for (int i = 0; i < rsModel.getColumnCount(); i++ )
		{
			if (columnChosen.equals(rsModel.getColumnName(i)))
			{
				if (i != selectedColumnIndex)
				{
					selectedColumnIndex = i;
					updatePieChart();
					return;
				}
				else
					return;
			}
		}
	}

}

public void chartMouseClicked(ChartMouseEvent event)
{
	chartMouseEventOccured(event, SELECT);
}

public void chartMouseMoved(ChartMouseEvent event)
{
	PieSectionEntity entity = (PieSectionEntity) event.getEntity();
	if (entity != null)
	{
		Object key = entity.getCategory();

		if (lastMouseOverKey != key)
		{	// Used to only fire mouseOver on mouseEnter events, not mouseMoved
			lastMouseOverKey = key;
			chartMouseEventOccured(event, MOUSEOVER);
		}
	}
	else
		lastMouseOverKey = null;
}

private void chartMouseEventOccured(ChartMouseEvent event, String type)
{
	PieSectionEntity entity = (PieSectionEntity) event.getEntity();
	if (entity != null)
	{
		Object key = entity.getCategory();
		Vector keys = new Vector();
		keys.add(key);
		SnapEvent snapEvent = new SnapEvent(this, type, keys);
		fireSnapEvent(snapEvent);
	}
}

/* This function will take in a String and will parse it into a double
 * as best that it can.  If it is unable, it will return 0.  */
protected double getValue(String number)
{
    String value;
    double answer = 0;
    if(number == null) return 0;

    value = number.replace('%', ' ').trim();
    try // try to parse the numeric format
    {
        answer = Double.parseDouble(value);
    }
    catch(NumberFormatException e)
    {
        if(value.length() == 0)
			answer = 0;
	else
	{
	    try  // try to parse everything before the first space
	    {
	        int space = value.indexOf(" ");
			if (space == -1)
				answer = 0;
			else
				answer = getValue(value.substring(0, space));
	    }
	    catch(Exception ex)
	    {
	        answer = 0;
	    }
	}
    }
    return answer;
}

/** Returns the component's Icon */
public Icon getIcon()
{
	return null;
}

/** Executes the appropriate action for the SnapEvent. */
public void performSnapEvent(SnapEvent e)
{
}

/** Applet's init method to create the content pane */
public void init()
{
	JLabel label = new JLabel("Empty PieChart");
	piePanel.add(label);
	allPanel.add(piePanel);
	allPanel.add(menuPanel, BorderLayout.SOUTH);
	this.setContentPane(allPanel);
}



}