package snap;

import snap.ui.*;
import snap.coordination.*;
import snap.database.*;
import snap.loader.*;
import snap.graph.*;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;
import java.sql.*;

/** Applet for coordinating the multiple components as a single,
  * multiple-view visualization.
  *
  * @author Nathan Conklin - nathan@conklinfamily.net
  * @author Varun Saini - vsaini@vt.edu
  * @author Chris North - north@cs.vt.edu
  * @author Umer Farooq - ufarooq@vt.edu
  * @author Aniket Prabhune - aprabhun@vt.edu
  * @author Matt Clement - macleme2@vt.edu
  */
public class Snap extends JApplet
{

protected boolean inAnApplet = true;  // distinguish from application

private SnapPanel feedbackPanel;
private CoordinationManager coordinationManager;
private Graph graph;
private DatabaseManager databaseManager;

private String host = "";

/** Builds the Snap content pane; initializes the database manager
  * and the frame manager */
protected Container makeContentPane()
{
    try
    {
        setHost(getCodeBase().getHost());
    }
    catch(Exception e)
    {
        setHost("");
    }

    JPanel pane = new JPanel();
    pane.setLayout(new BorderLayout());

    // Gets the pointer to the DatabaseManager Panel
    setDatabaseManager( new DatabaseManager(this));
    setFeedbackPanel(new SnapPanel(this));
	databaseManager.addConnectionChangeEventListener(getFeedbackPanel());
    feedbackPanel.setText("Choose a data source and press 'Connect'.");

	FrameManager frameManager = new FrameManager(this);
	databaseManager.addConnectionChangeEventListener(frameManager);

    try
    {
		// Enables communication with Java applets
		StaticMemoryList.setManager(this);
    }
    catch(Exception e)
    {
        getFeedbackPanel().setText("StaticMemoryList Error: " +
	    e.toString());
    }
    pane.add(feedbackPanel);
    return pane;
}

/* ********************************************************* */
/*    Accessor and Mutator methods                           */
/* ********************************************************* */

/** Returns the host that Snap has been loaded from.  This host is the
  * server that will hold the Visualization database.  */
public String getHost()
{
   return host;
}

/** Sets the host that Snap has been loaded from.  This host is the
  * server that will hold the Visualization database.  */
public void setHost(String host)
{
   this.host = host;
}

/** Returns the SnapPanel of the Snap user interface */
public SnapPanel getFeedbackPanel()
{
    return feedbackPanel;
}

/** Sets the SnapPanel of the Snap user interface */
public void setFeedbackPanel(SnapPanel feedbackPanel)
{
    this.feedbackPanel = feedbackPanel;
}

/** Sets the Graph used in coordination */
public void setGraph(Graph graph)
{
    this.graph = graph;
}

/** Returns the Graph used in coordination */
public Graph getGraph()
{
    return graph;
}

/** Sets the CoordinationManager that will handle coordination events. */
public void setCoordinationManager(CoordinationManager manager)
{
    coordinationManager = manager;
}

/** Returns the CoordinationManager that will handle coordination events. */
public CoordinationManager getCoordinationManager()
{
    return coordinationManager;
}

/** Sets the DatabaseManager */
public void setDatabaseManager(DatabaseManager manager)
{
    databaseManager = manager;
}

/** Returns the DatabaseManager */
public DatabaseManager getDatabaseManager()
{
    return databaseManager;
}


/* **************************************************************** */
/* Code to set up Snap as an applet or application                  */
/* **************************************************************** */

/** Constructor and applet initialization */
public Snap()
{
    inAnApplet = true;
}

/** Constructor and applet initialization.  inAnApplet is true if Snap
  * has been started as an applet */
public Snap(boolean inAnApplet)
{
    this.inAnApplet = inAnApplet;
    if(inAnApplet)
    {
        getRootPane().putClientProperty(
            "defeatSystemEventQueueCheck", Boolean.TRUE);
    }
}

/** Starts up the applet if it is not in a browser */
public static void main(String args[])
{
    JFrame frame = new JFrame("Snap");

    frame.addWindowListener(new WindowAdapter()
    {
        public void windowClosing(WindowEvent e)
		{
				System.exit(0);
		}
	});

    Snap applet = new Snap(false);
    frame.setContentPane(applet.makeContentPane());
    frame.pack();
    frame.setVisible(true);
}

/** JApplet's init() method. This should not be explicitly called. **/
public void init()
{
    setContentPane(makeContentPane());
}

};
