
package snap.adapter;

import snap.*;
import snap.database.*;
import snap.graph.*;

import java.sql.*;
import java.util.*;
import javax.swing.Icon;

/** General Adapter that defines Snap's general interface with a
  * visualization.
  *
  * @author Varun Saini - vsaini@vt.edu 
  * @author Chris North - north@cs.vt.edu
  * @author Nathan Conklin - nathan@conklinfamily.net
  * @author Umer Farooq - ufarooq@vt.edu
  * @author Aniket Prabhune - aprabhun@vt.edu
  */
public abstract class TechnologyAdapter
{

/* **************************************************************** */
/*  Abstract methods - must be implemented by extending classes     */
/* **************************************************************** */

/** Called when Snap needs to load data into the visualization */
abstract public void load(Query query);

/** Called when Snap needs to fire an event at the visualization */
abstract public void performSnapEvent(SnapEvent e);

/** Returns a list of Actions supported by a visualization */
abstract public Enumeration getSupportedActions();

/** Retusn the component's Icon */
abstract public Icon getIcon();

/* **************************************************************** */
/*  Concrete methods - implemented for the extending classes        */
/* **************************************************************** */

/** Container to keep track of the registered listeners */
private Vector listeners = new Vector();

/** TechnologyAdapater's node in the UI/Coordination Graph */
private Node node = null;

/** Sets the TechnologyAdapater's node in the UI/Coordination Graph */
public void setNode(Node node)
{
	this.node = node;
}

/** Returns the TechnologyAdapater's node in the UI/Coordination Graph.
    null is returned if the TechnologyAdapter has not been put in the 
graph. */
public Node getNode()
{
	return node;
}

/** Removes the listener for SnapEvents */
public void removeSnapEventListener(SnapEventListener sel)
{
    listeners.remove(sel);
}

/** Installs the listener for SnapEvents */
public void addSnapEventListener(SnapEventListener sel)
{
    listeners.add(sel);
}

/** Fires any SnapEvent that must be thrown by the component.*/
protected void fireSnapEvent(SnapEvent snapEvent)
{
    Enumeration e = listeners.elements();
    while(e.hasMoreElements())
    {
        SnapEventListener sel = (SnapEventListener) e.nextElement();
		sel.snapEventOccured(snapEvent);
    }
}

}

