
package snap.database;

import java.util.*;

/** Defines the database query that a visualization will wrap.
  *
  * @author Nathan Conklin - nathan@conklinfamily.net
  * @author Chris North - north@cs.vt.edu
  * @author Varun Saini - vsaini@vt.edu
  * @author Umer Farooq - ufarooq@vt.edu
  * @author Aniket Prabhune - aprabhun@vt.edu
  */
public class Query
{

private String tableName;
private Vector fields;
private String whereClause=null;

private String primaryKeyColumnName;

/** Constructor */
public Query(String tableName, Vector fields, String whereClause)
{
	setQuery(tableName, fields, whereClause);
}

/** Empty Constructor, sets an empty String for the Query */
public Query()
{
	setQuery("", new Vector(), "");
}

/** Constructor for creating a copy of the query. */
public Query(Query q)
{
	setQuery( q.getTable(), q.getFields(), q.getWhereClause());
}

public boolean equals(Query q)
{
	if(toString().equals(q.toString()))
		return true;
	else
		return false;
}

public boolean isEmpty()
{
	Query emptyQuery = new Query();
	if (toString().equals(emptyQuery.toString()))
		return true;
	else
		return false;
}

private String fieldsToString()
{
	String fields = "";
	Enumeration e = this.getFields().elements();
	while(e.hasMoreElements())
	{
		// Puts the String in brackets, protects agains fields with spaces
		fields += "[" + e.nextElement() + "]";
		if(e.hasMoreElements())
			fields += ", ";
	}
	return fields;
}

public String toString()
{
	if (getTable().equals(""))
	{
		return "";
	}
	else
	{
		String query = "SELECT ";
		query += fieldsToString() + " ";
		query += "FROM [" + this.getTable() + "] ";
		if(whereClause == null || whereClause.trim().equals(""))
			query += ";";
		else
			query += "WHERE " + whereClause + ";";
		return query;
	}
}

public void setQuery(String tableName, Vector fields, String whereClause)
{
	setTable(tableName);
	setFields(fields);
	setWhereClause(whereClause);
}

public String getTable()
{
	return tableName;
}

public void setTable(String tableName)
{
	this.tableName = tableName;
}

public Vector getFields()
{
	return fields;
}

public void setFields(Vector fields)
{
	this.fields = fields;
}

public String getWhereClause()
{
	return whereClause;
}

public void setWhereClause(String whereClause)
{
	this.whereClause = whereClause;
}

/** Adds a condition to the WHERE clause of the query.  The condition
  * should be in a form similar to "Age > 21" in the following SQL
  * statement: "SELECT Name,Age FROM Students WHERE Age > 21" */
public void addToWhereClause(String filter)
{
	if((whereClause == null) || (whereClause.equals(new String(""))))
		whereClause = filter;
	else
		whereClause += " AND " + filter;
}

}

