
package snap.graph;

import java.util.*;

/** This is the graph structure to manage the CoordinationNodes and their 
  * connections.  Each CoordinationNode stores the information about the 
  * visualization and the one directional link out from the node.
  *
  * @author Nathan Conklin - nathan@conklinfamily.net
  * @author Chris North - north@cs.vt.edu
  * @author Varun Saini - vsaini@vt.edu
  * @author Umer Farooq - ufarooq@vt.edu
  * @author Aniket Prabhune - aprabhun@vt.edu
  */
public class Graph
{

// keys - Visualization's unique name
// values - Coordination node for the visualization
private Hashtable nodes = new Hashtable();
private Monitor monitor = new Monitor();

// Container to keep track of the registered listeners
private Vector listeners = new Vector();

/** Removes the listener for GraphChangeEvents */
public void removeGraphChangeEventListener(GraphChangeEventListener gcel)
{
    listeners.remove(gcel);
}

/** Installs the listener for GraphChangeEvents */
public void addGraphChangeEventListener(GraphChangeEventListener gcel)
{
    listeners.add(gcel);
}

/** Fires a GraphChangeEvent */
protected void fireGraphChangeEvent(GraphChangeEvent gce)
{
	Enumeration e = listeners.elements();
	while(e.hasMoreElements())
	{
		GraphChangeEventListener listener;
		listener = (GraphChangeEventListener) e.nextElement();
		listener.graphChanged(gce);
	}
}

/** Returns the monitor for locking the entire graph */
public Monitor getMonitor()
{
	return monitor;
}

public Node getNode(String name)
{
	return (Node) nodes.get(name);
}

public void setNode(String name, Node node)
{
	nodes.put(name, node);
}

public Node removeNode(String name)
{
	return (Node) nodes.remove(name);
}

public Enumeration nodes()
{
	if(nodes == null)
	{
		Vector v = new Vector();
		return v.elements();
	}
	else
		return nodes.elements();
}

}
