
package snap.loader.cave;

import snap.loader.*;
import snap.adapter.*;
import snap.graph.*;
import snap.*;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.util.Enumeration;

/** An applet that runs on the same page as a Java Applet visualization
  * component.  The AppletLoader with find the applet on the page and will
  * register that component with Snap.
  *
  * @author Nathan Conklin - nathan@conklinfamily.net
  * @author Chris North - north@cs.vt.edu
  */
public class CaveLoader extends JApplet
{

protected boolean inAnApplet = true;  // distinguish from application
protected JTextArea textArea;
protected String moleculeName = null;
protected String atomName = null;
protected String bondName = null;


private Snap snap;
//private JavaTechnologyAdapter adapter;


/** Finds and loads a Snapable applet */
public void start()
{
	//System.out.println("start()");
	snap = (Snap) StaticMemoryList.getManager();
	UIGraph graph = (UIGraph) snap.getGraph();

	CaveMoleculeAdapter moleculeAdapter = new CaveMoleculeAdapter(snap);
	moleculeName = graph.addComponent("Molecule Adapter", moleculeAdapter);

	CaveAtomAdapter atomAdapter = new CaveAtomAdapter(snap);
	atomName = graph.addComponent("Atom Adapter", atomAdapter);

	CaveBondAdapter bondAdapter = new CaveBondAdapter(snap);
	bondName = graph.addComponent("Bond Adapter", bondAdapter);
	snap.getFeedbackPanel().setText("All adapters created sucessfully");
}


/** Unloads the Snapable applet */
public void stop()
{
	if(atomName != null || moleculeName != null || bondName != null)
	{
		try
		{
			UIGraph graph = (UIGraph) snap.getGraph();
			graph.removeComponent(atomName);
graph.removeComponent(moleculeName);
graph.removeComponent(bondName);
		}
		catch(Exception e)
		{
			String error =
				"Error talking to Snap or trying to unload Cave Components - " +
				e.toString();
			System.err.println(error);
		}
	}
}

/** Returns the AppletInfo for the AppletLoader */
public String getAppletInfo()
{
    return "Cave Loader";
}

/* **************************************************************** */
/* Code to set up AppletLoader as an applet or application          */
/* **************************************************************** */

/** JApplet's init() method. This should not be explicitly called. **/
public void init()
{
    setContentPane(makeContentPane());
}

/** Builds the AppletLoader content pane with a TextArea that may be
used for basic output. */
protected Container makeContentPane()
{
    JPanel pane = new JPanel();
    pane.setLayout(new BorderLayout());
    textArea =
		new JTextArea("Hello world");
    JScrollPane scrollPane = new JScrollPane(textArea);
    scrollPane.add(pane);
    scrollPane.setBorder(new EmptyBorder(0, 0, 0, 0));
    return scrollPane;
}

/** Constructor and applet initialization */
public CaveLoader()
{
    inAnApplet = true;
}

/** Constructor and applet initialization.  inAnApplet is true if
AppletLoader has been started as an applet */
public CaveLoader(boolean inAnApplet)
{
    this.inAnApplet = inAnApplet;
    if(inAnApplet)
    {
        getRootPane().putClientProperty(
            "defeatSystemEventQueueCheck", Boolean.TRUE);
    }
}

/** Starts up the applet if it is not in a browser */
public static void main(String args[])
{
    JFrame frame = new JFrame("AppletLoader");

    frame.addWindowListener(new WindowAdapter()
    {
        public void windowClosing(WindowEvent e)
	{
            System.exit(0);
	}
    }
    );

    CaveLoader applet = new CaveLoader(false);
    frame.setContentPane(applet.makeContentPane());
    frame.pack();
    frame.setVisible(true);
}

};
