
package snap.ui;

import snap.*;
import snap.database.*;
import snap.ui.diagram.*;
import snap.coordination.*;
import snap.graph.*;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import javax.swing.*;
import javax.swing.border.*;
import java.sql.*;

/** The basic UI for Snap.
  *
  * @author Nathan Conklin - nathan@conklinfamily.net
  * @author Chris North - north@cs.vt.edu
  */
public class SnapPanel extends JPanel
implements ConnectionChangeEventListener
{

protected Snap snap;
protected VisualizationSchema visualizationSchema;

/* UI objects - indented to show the containment */
protected JPanel titlePanel = new JPanel();
    protected JLabel title = new JLabel("Snap");
    protected JPanel feedbackPanel = new JPanel();
        protected JTextArea text = new JTextArea();
        protected JScrollPane scrollPane;
protected JTabbedPane tabbedPane = new JTabbedPane(SwingConstants.TOP);

/* Tab titles and descriptions */
protected String databaseConfigurationTitle = "Database";
protected String databaseConfigurationComment =
			"Connects Snap to the data source.";
protected String visualizationSchemaTitle = "Visualization Schema";
protected String visualizationSchemaComment =
			"Snaps together different visualizations.";

/** Creates or removes a VisualizationSchema. */
public void connectionChanged(ConnectionChangeEvent e)
{
	DatabaseManager databaseManager = (DatabaseManager) e.getSource();
    if(databaseManager.getConnection() != null)
	{
		// Sets up the CoordinationManager
		snap.setGraph(new UIGraph());
		CoordinationManager cm = new CoordinationManager(snap);
		snap.setCoordinationManager(cm);
		snap.getGraph().addGraphChangeEventListener(cm);

		// Add a new tab with the VisualizationSchema
		removeVisualizationSchemaTab();
        visualizationSchema = new VisualizationSchema(snap);
        tabbedPane.addTab(visualizationSchemaTitle, null, visualizationSchema,
            visualizationSchemaComment);
		tabbedPane.setSelectedComponent(visualizationSchema);
    }
    else // Remove an existing VisualizationSchema
    {
		snap.setGraph(null);
		snap.setCoordinationManager(null);
		removeVisualizationSchemaTab();
    }
}

private void removeVisualizationSchemaTab()
{
	int index;
	index = tabbedPane.indexOfTab(visualizationSchemaTitle);
	if(index >= 0)
		tabbedPane.remove(index);
		visualizationSchema = null;
}

/** Returns the text for the status. */
public String getText()
{
    return text.getText();
}

/** Updates the user about the status. @param text is the String that
will be displayed to the user. **/
public void setText(String text)
{
	System.out.println(text);
    this.text.setText(text);
}

/** Constructor for the SnapPanel.  @param applet is a reference to
the Snap object **/
public SnapPanel(Snap snap)
{
    this.snap = snap;
	jbInit();
}

protected void jbInit()
{

    // Set up the title
    title.setFont(new java.awt.Font("Dialog", 1, 12));
    title.setForeground(Color.black);
    //title.setBorder(BorderFactory.createEmptyBorder(0,0,0,0));
    title.setBorder(BorderFactory.createLineBorder(Color.black));
    title.setHorizontalAlignment(SwingConstants.CENTER);
    title.setLabelFor(this);

    titlePanel.setLayout(new BorderLayout());
    titlePanel.add(title, BorderLayout.NORTH);

    // Set up the feedback section
    text.setWrapStyleWord(true);
    text.setLineWrap(true);
    text.setBackground(Color.white);
    scrollPane = new JScrollPane(text);
    scrollPane.setPreferredSize(new Dimension(0, 75));
    titlePanel.add(scrollPane, BorderLayout.CENTER);

    // ------------------------------------
    // Adding components to the JTabbedPane
    // ------------------------------------
    tabbedPane.addTab(databaseConfigurationTitle, null,
		snap.getDatabaseManager(), databaseConfigurationComment);
    tabbedPane.setSelectedIndex(0);

    // Pull together the title and the control panel
    this.setBorder(BorderFactory.createLineBorder(Color.black));
    this.setLayout(new BorderLayout());
    this.add(titlePanel, BorderLayout.NORTH);
    this.add(tabbedPane, BorderLayout.CENTER);
}

};
