package snap.ui.diagram;

/**
  * Coordination is the pair of actions that are coordinated between
  * two visualizations.
  *
  * @author Nathan Conklin - nathan@conklinfamily.net
  * @author Matt Clement - macleme2@vt.edu
  * @author Chris North - north@cs.vt.edu
  */
public class Coordination
{

private String		startAction,	//start pt. of conneciton
					endAction;	//end pt. of connection

private Link		link; // link that the coordination is associated with


public Coordination(Link link)
{
	this("", "", link);
}

/**
 * Class constructor
 * 
 * @param startAction	the visualization where the connection starts
 * @param endAction		the ending point of the UI representation
 * @param link			the link that this Coordination is part of
 *
 */
public Coordination(String startAction, String endAction, Link link)
{
	this.startAction = startAction;
	this.endAction = endAction;
	this.link = link;
}


public String getStartAction()
{
	return startAction;
}

public void setStartAction(String action)
{
	startAction = action;
}

public String getEndAction()
{
	return endAction;
}

public void setEndAction(String action)
{
	endAction = action;
}

public String getName()
{
	String name;
	String startViz = link.getStartVisualization().getUINode().getName();
	String endViz = link.getEndVisualization().getUINode().getName();

	if(startAction.equals("") && endAction.equals(""))
	{
		// Vis -- Vis
		name = startViz + " -- " + endViz;
	}
	else
	{
		// Vis->action -- Vis->action
		name = startViz + "->" + startAction + " -- " +
			endViz + "->" +	endAction;
	}
	return name;
}

}
