
package table;

import java.util.*;
import java.awt.*;
import java.awt.Font;
import java.awt.event.*;
import javax.swing.event.*;
//import javax.swing.JComponent;
import java.applet.*;
import javax.swing.*;
import java.sql.*;
import java.net.URL;

import snap.*;

/** Example of a JTable as a Snapplet.
  * @author Nathan Conklin - nathan@conklinfamily.net
  * @author Chris North - north@cs.vt.edu
  */
public class TableSnapplet extends Snapplet
implements Snapable
{

private boolean inAnApplet = true;
private boolean DEBUG = true;

private ResultSet rs = null;
private String primaryKeyColumnName = null;
private int keyColumn;

private JPanel panel = new JPanel();
private JScrollPane scrollPane = null;

private ResultSetTableModel myModel = null;
private TableSorter sorter = null;
private CAVE_table table = new CAVE_table();

//private Font Tfont = new Font ("bigT", 1, 16);
//table.setFont(Tfont);

private String APPLET_TYPE = "JTable with Sorting";
private String SELECT_EVENT_TYPE = "Select";
private String SCROLL_EVENT_TYPE = "Scroll (receive only)";
private int DEFAULT_PRIMARY_KEY_INDEX = 0;

/** Returns an Enumeration of SnapEvents that the table can perform
  * and will fire */
public Enumeration getSupportedActions()
{
	Vector v = new Vector();
	v.add( SELECT_EVENT_TYPE );
	v.add( SCROLL_EVENT_TYPE );
	return v.elements();
}

/** Implementation of Snapable's load() method */
public void load(ResultSet rs, String primaryKeyColumnName)
{
    this.rs = rs;
    this.primaryKeyColumnName = primaryKeyColumnName;

    myModel = new ResultSetTableModel(rs);
    sorter = new TableSorter(myModel);
    table.setModel(sorter);
    sorter.addMouseListenerToHeaderInTable(table, findColumn(primaryKeyColumnName), this);
}

/** Returns the component's Icon */
public Icon getIcon()
{
	Icon icon = null;
	try
	{
		String base = getCodeBase().toString();
		String iconURL = base + "table/table.gif";
		icon = new ImageIcon(new URL(iconURL));
	}
	catch(Exception e)
	{
		System.err.println("TableSorterDemo: " +
			"Unable to create an icon: " + e.toString());
	}
	return icon;
}

/** Returns the Snapplet's type info */
public String getAppletInfo()
{
	return APPLET_TYPE;
}

/** Executes the appropriate action for the SnapEvent.
  * Table only supports "Select" event. */
public void performSnapEvent(SnapEvent e)
{
	keyColumn = findColumn(primaryKeyColumnName);
	Vector keys = e.getKeys();

    if(e.getEventType().equals(SELECT_EVENT_TYPE))
    {
		select(keys);
		scroll(keys);
    }
	else if(e.getEventType().equals(SCROLL_EVENT_TYPE))
    {
		scroll(keys);
    }
}

protected void select(Vector keys)
{
	ListSelectionModel rowSM = table.getSelectionModel();
	rowSM.clearSelection();

	if(keys != null)  // Nothing is to be selected if null
	{	// Iterate across the data elements
		for(int i = 0; i < sorter.getRowCount(); i++)
		{
			Object o = sorter.getValueAt(i, keyColumn);
			Enumeration keysEnum = keys.elements();
			boolean selected = false;

			// Iterate across the selected keys
			while(keysEnum.hasMoreElements() && selected == false)
			{
				Object key = keysEnum.nextElement();
				if(o.equals(key))
				{	// Adds the row to the set of rows that need to select
					rowSM.addSelectionInterval(i, i);
					selected = true;
				}
			}
		}
	}
	repaint();
}

protected void scroll(Vector keys)
{
	ListSelectionModel rowSM = table.getSelectionModel();
	int firstRowSelected = 0;
	int lastRowSelected = 0;

	if(keys != null)
	{	// Iterate across the data elements
		for(int i = 0; i < sorter.getRowCount(); i++)
		{
			Object o = sorter.getValueAt(i, keyColumn);
			Enumeration keysEnum = keys.elements();

			// Iterate across the selected keys
			while(keysEnum.hasMoreElements())
			{
				Object key = keysEnum.nextElement();
				if(o.equals(key))
				{
					// Adds the row to the set of rows that need to show
					if (firstRowSelected == 0)
						firstRowSelected = i;
					lastRowSelected = i;
				}
			}
		}
		// Scroll to the newly selected rows
		makeRowsVisible(table, firstRowSelected, lastRowSelected);
		repaint();
	}


}

/**	This function will scroll to the first selected row
private void scroll()
{
	ListSelectionModel rowSM = table.getSelectionModel();

	for(int i = 0; i < sorter.getRowCount(); i++)
	{
		if (rowSM.isSelectedIndex(i))
		{
			table.scrollRectToVisible(table.getCellRect(i, 1, true));
			break;
		}
	}
	repaint();
}
*/


/** Finds the column index for the primaryKeyColumnName */
private int findColumn(String primaryKeyColumnName)
{
    int keyColumn;
    if(primaryKeyColumnName != null)
    {
        keyColumn = sorter.findColumn(primaryKeyColumnName);
        if(keyColumn < 0)   // Error finding the column
            keyColumn = DEFAULT_PRIMARY_KEY_INDEX; // First column
    }
    else
	{
        keyColumn = DEFAULT_PRIMARY_KEY_INDEX;  // Default to first column
	}
    return keyColumn;
}

/* Installs the selection listener.  Will fire the appropriate
 * SnapEvent when rows in the table are selected */
private void setSelectionListener()
{
    ListSelectionModel rowSM = table.getSelectionModel();
    rowSM.addListSelectionListener(new ListSelectionListener() {
        public void valueChanged(ListSelectionEvent e)
		{
            //Ignore extra messages.
            if (e.getValueIsAdjusting()) return;

            keyColumn = findColumn(primaryKeyColumnName);

			SnapEvent snapEvent;
			ListSelectionModel lsm = (ListSelectionModel)e.getSource();
            if (lsm.isSelectionEmpty())
			{
				snapEvent = new SnapEvent(this, SELECT_EVENT_TYPE, null);
            }
			else
			{
				int minSelectedRow = lsm.getMinSelectionIndex();
				int maxSelectedRow = lsm.getMaxSelectionIndex();
				Vector keys = new Vector();

				for(int i = minSelectedRow; i <= maxSelectedRow; i++)
				{
					if(lsm.isSelectedIndex(i))
					{
						keys.add(sorter.getValueAt(i, keyColumn));
					}
				}
				snapEvent = new SnapEvent(this, SELECT_EVENT_TYPE, keys);
            }
            fireSnapEvent(snapEvent);
        }
    });
}



/** Builds the applet's content pane that will be displayed */
private Container makeContentPane()
{
    panel.setLayout(new BorderLayout());

    load(null, null);
    table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
    setSelectionListener();

    scrollPane = new JScrollPane(table);
    scrollPane.revalidate();

    panel.add(scrollPane, BorderLayout.CENTER);

    return panel;
}

/** Constructor used with the component as an applications */
public TableSnapplet(boolean inAnApplet)
{
    this.inAnApplet = inAnApplet;
    if(inAnApplet)
    {
        getRootPane().putClientProperty(
            "defeatSystemEventQueueCheck", Boolean.TRUE);
    }
}

/** Default constructor with component as an applet */
public TableSnapplet()
{
	this(true);
}

/** Applet's init method to create the content pane */
public void init()
{
    setContentPane(makeContentPane());
}

/** Application's main method that creates a table in a JFrame */
public static void main(String[] args)
{
	JFrame frame = new JFrame("Table");

	frame.addWindowListener(new WindowAdapter()
	{
		public void windowClosing(WindowEvent e)
		{
			System.exit(0);
		}
	});

	TableSnapplet applet = new TableSnapplet(false);

	frame.setContentPane(applet.makeContentPane());
	frame.pack();
	frame.setVisible(true);
}

/** ******************************************************************** */
/** Methods used in scrolling                                            */
/** ******************************************************************** */

public static Rectangle getRowBounds(JTable table, int row)
{
	checkRow(table, row);

	Rectangle result = table.getCellRect(row, -1, true);
	Insets i = table.getInsets();

	result.x = i.left;
	result.width = table.getWidth() - i.left - i.right;

	return result;
}

public static Rectangle getRowBounds(JTable table, int first, int last)
{
	checkRows(table, first, last);

	Rectangle result = table.getCellRect(first, -1, true);
	result = result.union(table.getCellRect(last, -1, true));
	Insets i = table.getInsets();

	result.x = i.left;
	result.width = table.getWidth() - i.left - i.right;

	return result;
}


public static Rectangle getColumnBounds(JTable table, int column)
{
	checkColumn(table, column);

	Rectangle result = table.getCellRect(-1, column, true);
	Insets i = table.getInsets();

	result.y = i.top;
	result.height = table.getHeight() - i.top - i.bottom;

	return result;
}

public static Rectangle getColumnBounds(JTable table, int first, int last)
{
	checkColumns(table, first, last);

	Rectangle result = table.getCellRect(-1, first, true);
	result = result.union(table.getCellRect(-1, last, true));
	Insets i = table.getInsets();

	result.y = i.top;
	result.height = table.getHeight() - i.top - i.bottom;

	return result;
}


/** For completeness. Only allows valid rows/columns. */
public static Rectangle getCellBounds(JTable table, int row, int column)
{
	checkCell(table, row, column);

	return table.getCellRect(row, column, true);
}

public static Rectangle getCellBounds(
	JTable table, int firstRow, int lastRow, int firstColumn, int lastColumn)
{
	checkCells(table, firstRow, lastRow, firstColumn, lastColumn);

	Rectangle result = table.getCellRect(firstRow, firstColumn, true);
	return result.union(table.getCellRect(lastRow, lastColumn, true));
}


public static void makeRowVisible(JTable table, int row)
{
	Scrolling.scrollVertically(table, getRowBounds(table, row));
}


public static void makeColumnVisible(JTable table, int column)
{
	Scrolling.scrollHorizontally(table, getColumnBounds(table, column));
}


public static void makeRowsVisible(JTable table, int first, int last)
{
	Scrolling.scrollVertically(table, getRowBounds(table, first, last));
}


public static void makeRowsVisible(JTable table, int first, int last, int bias)
{
	Scrolling.scrollVertically(table, getRowBounds(table, first, last), bias);
}

public static void makeColumnsVisible(JTable table, int first, int last)
{
	Scrolling.scrollHorizontally(table, getColumnBounds(table, first, last));
}

public static void makeColumnsVisible(
	JTable table, int first, int last, int bias)
{
	Scrolling.scrollHorizontally(
		table, getColumnBounds(table, first, last), bias);
}

public static void makeCellsVisible(
	JTable table, int firstRow, int lastRow, int firstColumn, int lastColumn)
{
	table.scrollRectToVisible(
		getCellBounds(table, firstRow, lastRow, firstColumn, lastColumn));
}

public static void makeCellsVisible(JTable table, int firstRow, int lastRow,
	int firstColumn, int lastColumn, int bias)
{
	Scrolling.scroll(table, getCellBounds(
		table, firstRow, lastRow, firstColumn, lastColumn), bias);
}


public static void makeCellsVisible(JTable table, int firstRow, int lastRow,
	int firstColumn, int lastColumn, int rowBias, int columnBias)
{
	Scrolling.scroll(
		table,
		getCellBounds(table, firstRow, lastRow, firstColumn, lastColumn),
		rowBias,
		columnBias);
}


public static void centerRow(JTable table, int row)
{
	Scrolling.centerVertically(table, getRowBounds(table, row), false);
}


public static void centerColumn(JTable table, int column)
{
	Scrolling.centerHorizontally(table, getColumnBounds(table, column), false);
}


public static void centerRows(JTable table, int first, int last)
{
	Scrolling.centerVertically(table, getRowBounds(table, first, last), false);
}

public static void centerColumns(JTable table, int first, int last)
{
	Scrolling.centerHorizontally(
		table, getColumnBounds(table, first, last), false);
}

public static void centerCell(JTable table, int row, int column)
{
	Scrolling.center(table, getCellBounds(table, row, column), false);
}

public static void centerCells(JTable table, int firstRow, int lastRow,
	int firstColumn, int lastColumn)
{
	Scrolling.center(
		table,
		getCellBounds(table, firstRow, lastRow, firstColumn, lastColumn),
		false);
}


public static boolean isRowVisible(JTable table, int row)
{
	return Scrolling.isVerticallyVisible(table, getRowBounds(table, row));
}

public static boolean isColumnVisible(JTable table, int column)
{
	return Scrolling.isHorizontallyVisible(
		table, getColumnBounds(table, column));
}

public static boolean isCellVisible(JTable table, int row, int column)
{
	return Scrolling.isVisible(table, getCellBounds(table, row, column));
}

public static boolean areColumnsVisible(JTable table, int first, int last)
{
	return Scrolling.isHorizontallyVisible(
		table, getColumnBounds(table, first, last));
}

public static boolean areRowsVisible(JTable table, int first, int last)
{
	return Scrolling.isVerticallyVisible(
		table, getRowBounds(table, first, last));
}

public static boolean areCellsVisible(
	JTable table, int firstRow, int lastRow, int firstColumn, int lastColumn)
{
	checkCells(table, firstRow, lastRow, firstColumn, lastColumn);

	return Scrolling.isVisible(
		table,
		getCellBounds(table, firstRow, lastRow, firstColumn, lastColumn));
}



private static void checkRow(JTable table, int row)
{
	if (row < 0)
		throw new IndexOutOfBoundsException(row+" < 0");
	if (row >= table.getRowCount())
		throw new IndexOutOfBoundsException(row+" >= "+table.getRowCount());
}

private static void checkColumn(JTable table, int column)
{
	if (column < 0)
		throw new IndexOutOfBoundsException(
			column+" < 0");
	if (column >= table.getColumnCount())
		throw new IndexOutOfBoundsException(
			column + " >= " + table.getColumnCount());
}

private static void checkCell(JTable table, int row, int column)
{
	checkRow(table, row);
	checkColumn(table, column);
}


private static void checkRows(JTable table, int first, int last)
{
	if (first < 0)
		throw new IndexOutOfBoundsException(first+" < 0");
	if (first > last)
		throw new IndexOutOfBoundsException(first+" > "+last);
	if (last >= table.getRowCount())
		throw new IndexOutOfBoundsException(last+" >= "+table.getRowCount());
}


private static void checkColumns(JTable table, int first, int last)
{
	if (first < 0)
		throw new IndexOutOfBoundsException(first+" < 0");
	if (first > last)
		throw new IndexOutOfBoundsException(first+" > "+last);
	if (last >= table.getColumnCount())
		throw new IndexOutOfBoundsException(last+" >= "+table.getColumnCount());
}


private static void checkCells(
	JTable table, int firstRow, int lastRow, int firstColumn, int lastColumn)
{
	checkRows(table, firstRow, lastRow);
	checkColumns(table, firstColumn, lastColumn);
}

/** ******************************************************************** */
/** End of methods used in scrolling                                     */
/** ******************************************************************** */

}

