#include "FakeMouse.h"

FakeMouse::FakeMouse(char *disp){
  char* display = disp;
  
  moving_wnd = 0;
  resizing_wnd = 0;
  drawing_line = 0;

  if ((dpy = XOpenDisplay(display)) == NULL){
    fprintf(stderr,
	    "%s Error: XOpenDisplay(\"%s\"): unable to open display: %s\n",
	    disp, display, display);
    exit(1);
  }
}

FakeMouse::~FakeMouse()
{
	printf("Destroying fake mouse\n");
}

void FakeMouse::update(){
  last_x = real_x;
  last_y = real_y;
  XQueryPointer(dpy, DefaultRootWindow(dpy), &root, &current_wnd,
		&real_x, &real_y, &wnd_x, &wnd_y, &mask);

  //if currently moving a window... do so
  if(drawing_line==1){
    fprintf(stderr, "Drawing Line\n");
  }

  //if the mouse is in a window get it's attributes
  if(current_wnd){
    XGetWindowAttributes(dpy, current_wnd, &attrib);
  }

}

void FakeMouse::Press(int b){
    XW_send_fake_Xbutton(dpy,b,True);  /* press button */
}

void FakeMouse::Release(int b){
    XW_send_fake_Xbutton(dpy,b,False);  /* press button */
}

void FakeMouse::Click(int b){
  Press(b);
  Release(b);
}

void FakeMouse::DoubleClick(int b){
  Click(b);
  Click(b);
}

void FakeMouse::SetPosition(int x, int y){
  XWarpPointer(dpy, None, (DefaultRootWindow(dpy)), 0,0,0,0, x, y);
  XFlush(dpy);
}

void FakeMouse::AddPosition(int x, int y){
  SetPosition(real_x+x, real_y+y);
}

void FakeMouse::RaiseWindow(){
  if(current_wnd){
    XRaiseWindow(dpy, current_wnd);
    XGetWindowAttributes(dpy, current_wnd, &attrib);
  }
}

void FakeMouse::LowerWindow(){
  if(current_wnd){
    XLowerWindow(dpy, current_wnd);
    XGetWindowAttributes(dpy, current_wnd, &attrib);
  }
}

void FakeMouse::StartResizeWindow(){
  XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, XK_Alt_L),1,CurrentTime);
  XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, XK_F8),1,CurrentTime);
  XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, XK_Alt_L),0,CurrentTime);
  XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, XK_F8),0,CurrentTime);
}

void FakeMouse::StopResizeWindow(){
  Click(1);
}

void FakeMouse::StartMoveWindow(){
  XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, XK_Alt_L),1,CurrentTime);
  XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, XK_F7),1,CurrentTime);
  XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, XK_Alt_L),0,CurrentTime);
  XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, XK_F7),0,CurrentTime);
}

void FakeMouse::StopMoveWindow(){
  Click(1);
}

void FakeMouse::StartDrawLine(){
  drawing_line = 1;
}

void FakeMouse::StopDrawLine(){
  drawing_line = 0;
}

void FakeMouse::DestroyWindow(){
  if(current_wnd){
    XDestroyWindow(dpy, current_wnd);
  }
}

void FakeMouse::MinimizeWindow(){
    fprintf(stderr, "MinimizeWindow!!!\n");
    if(current_wnd){
        int n = XIconifyWindow(dpy, current_wnd, 0);
        //int n = XWindow(dpy, current_wnd, 0);
        fprintf(stderr, "current window %d\n", n);
        XGetWindowAttributes(dpy, current_wnd, &attrib);
    }
}

void FakeMouse::MaximizeWindow(){
  XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, XK_Alt_L),1,CurrentTime);
  XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, XK_F10),1,CurrentTime);
  XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, XK_Alt_L),0,CurrentTime);
  XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, XK_F10),0,CurrentTime);
}

void FakeMouse::RestoreWindow(){
  XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, XK_Alt_L),1,CurrentTime);
  XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, XK_F5),1,CurrentTime);
  XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, XK_Alt_L),0,CurrentTime);
  XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, XK_F5),0,CurrentTime);
}

void FakeMouse::xtestext1_correct_mouse(Display *dpy){
  Window root, w;
  int rx, ry, wx, wy;
  unsigned int m;
  
  XQueryPointer(dpy, DefaultRootWindow(dpy), &root, &w,
		&rx, &ry, &wx, &wy, &m);
}


void FakeMouse::XW_send_fake_Xbutton(Display *dpy, int button, Bool press)
{
  XTestFakeButtonEvent(dpy,button,press,CurrentTime); /* button press now */
  xtestext1_correct_mouse(dpy);
}

void FakeMouse::get_pointer_position(){
  XQueryPointer(dpy, DefaultRootWindow(dpy), &root, &current_wnd,
		&real_x, &real_y, &wnd_x, &wnd_y, &mask);
}

int* FakeMouse::getMousePosition() {
  get_pointer_position();
  int* p = new int[2];
  p[0] = real_x;
  p[1] = real_y;
  return p;
}

void FakeMouse::doEvent(int event, int change, char* command){
    int keysym;
    
    switch (event) {
        case FM_BUTTON1:
            if(change==1){
                Press(1);
            }
            else {
                Release(1);
            }
            break;
        case FM_BUTTON2:
            if(change==1){
                Press(2);
            }
            else {
                Release(2);
            }
            break;
        case FM_BUTTON3:
            if(change==1){
                Press(3);
            }
            else {
                Release(3);
            }
            break;
        case FM_RAISE_WINDOW:
            if(change==1){
                //nothing
            }
            else {
                RaiseWindow();
            }
            break;
        case FM_LOWER_WINDOW:
            if(change==1){
                //nothing
            }
            else {
                LowerWindow();
            }
            break;
        case FM_MOVE_WINDOW:
            if(change==1){
                StartMoveWindow();
            }
            else {
                StopMoveWindow();
            }
            break;
        case FM_RESIZE_WINDOW:
            if(change==1){
                StartResizeWindow();
            }
            else {
                StopResizeWindow();
            }
            break;
        case FM_MAXIMIZE_WINDOW:
            if(change==1){
                //nothing
            }
            else {
                MaximizeWindow();
            }
            break;
        case FM_MINIMIZE_WINDOW:
            if(change==1){
                //nothing
            }
            else {
                MinimizeWindow();
            }
            break;
        case FM_RESTORE_WINDOW:
            if(change==1){
                //nothing
            }
            else {
                RestoreWindow();
            }
            break;
        case FM_DESTROY_WINDOW:
            if(change==1){
                //nothing
            }
            else {
                DestroyWindow();
            }
            break;
        case FM_KEYPRESS:
            //get the keysym
            fprintf(stderr, "doing keypress %s\n", command);
            
            keysym = XStringToKeysym(command);
            if(keysym == NoSymbol){
                fprintf(stderr, "didn't find a symbol :::%s:::\n", command);
                break;
            }
            if(change==1){
                XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, keysym),1,CurrentTime);
            }
            else {
                XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, keysym),0,CurrentTime);
            }
            break;
        case FM_SYSTEM:
            if(change==1){
                //nothing
            }
            else {
                system(command);
            }
            break;
    }
    XFlush(dpy);
}

void FakeMouse::keyPress(int status, int keysym)
{
    XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, keysym), status, CurrentTime);
    XFlush(dpy);
}
