//
// DIVERSE graphics interface -- Performer ---- version 2.1.0
// released: Wed Jun 26 17:36:44 EDT 2002
// The DIVERSE HOMPAGE is at: www.diverse.vt.edu
// 
// Copyright (C) 2000, 2001, 2002  John Kelso, Lance Arsenault and Ron Kriz
// 
// This software, DIVERSE graphics interface -- Performer (DPF),
// is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License (LGPL) as
// published by the Free Software Foundation; either version 2.1 of
// the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this software, in the top level source directory in
// a file named "COPYING"; if not, see it at:
// http://www.gnu.org/copyleft/lesser.html or write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//
//
#include <stdio.h>
#include <dpf.h>
#include <dtk/dtkDSO_loader.h>
#include <string>
using namespace std;
#include "CommandParser.h"
#include "XWandCommands.h"

// example generic dtkAugment DSO modified to be an XWand dso!
// Notice how simple it is to take an example program and modify it

class Generic : public dtkAugment {
public:

  Generic(dpf *app) ;
  ~Generic(void) ;
  //During the post frame step handle the navigation
  int postFrame(void);

private:

  //The application to play with
  dpf* App;
  //The Command parser that gets data from the message stream
  CommandParser* GetMessage;
  //The command we got
  string Command;
  //The old command we had
  string OldCommand;
} ;

// create object and set description.
// be sure to validate it!
Generic::Generic(dpf *app) : dtkAugment("xwand-dso")
{
	Command = "";
	OldCommand = "";
	validate();
	App = app;
	GetMessage = new CommandParser("XWAND-DSO");
} 

// destructor
Generic::~Generic(void) 
{
 	delete GetMessage;
}




// called every display loop, as a pfFrame callback, after pfDraw()
// this return value causes it to only be called once, and the object removed
int Generic::postFrame(void) 
{
	GetMessage->getCommand(Command, CLASS_TYPE);
	if (OldCommand == Command)
		return DTK_CONTINUE;
	OldCommand = Command;
	//Start navigation
	if (Command == STARTNAV)
	{
		dpfPipe *p ;
		dpfPwin *w ;
		pfPipeWindow *pw ;
		for (p=App->display()->first(); (p); p=p->next())
		{
			for (w=p->first(); (w); w=w->next())
			{
				pw = w->pfpwin() ;
				pfuLoadPWinCursor(pw,PFU_CURSOR_OFF) ;
			}
		}
		App->display()->currentNav->activate();
	}
	//Stop navigation
	if (Command == STOPNAV)
	{
		dpfPipe *p ;
		dpfPwin *w ;
		pfPipeWindow *pw ;
		for (p=App->display()->first(); (p); p=p->next())
		{
			for (w=p->first(); (w); w=w->next())
			{
				pw = w->pfpwin() ;
				pfuLoadPWinCursor(pw,PFU_CURSOR_X) ;
			}
		}
		App->display()->currentNav->deactivate();

	}
  return DTK_CONTINUE ; 
}


/*************** DTK C++ DSO loader/unloader functions ***********/
/*
 * All DTK DSO files are required to declare these two functions.
 * These function are called by the loading program to get your
 * C++ objects loaded.
 *
 *****************************************************************/

static dtkAugment *dtkDSO_loader(dtkManager *manager, void *p)
{
  dpf *app = static_cast<dpf *>(manager);
  return new Generic(app);
}

static int dtkDSO_unloader(dtkAugment *augment)
{
  delete augment;
  return DTKDSO_UNLOAD_CONTINUE;
}
