#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "xwandparser.h"

//This file holds functions that parse the config file
//Basically a switch on the first char of the file and hten
//fill a struct up with data and send it back
//
//Function prototypes
void XWandParser(char* File); //Parse the file
void WhenPressed();           //Handle a when pressed token
void TriggerButton();         //Handle a trigger button token
void StartApp();              //Handle a startapp token

//Global vars
XData Data; //For ease of use, holds the data to be returned
//End Global variables

//Wrapping function for parsing the xwand
//This basically allows me to print out debug information if
//I need to
XData XParse(char* Filename, const bool& Verbose)
{
	XWandParser(Filename);
	if (Verbose)
	{
		printf("At the end of parsing here is the structure\n");
		printf ("Path = %s\n", Data.Path);
		printf("Num Buttons = %d\n", Data.NumButtons);
		for (int i=0; i < Data.NumButtons; i++)
		{
			printf("Button %d is needed to activate the program\n", Data.Buttons[i]);
		}
		printf("Displaying on %s\n", Data.Display);
		if (Data.InDir)
			printf("Displaying in the current directory\n");
		else
			printf("Not displaying in the current directory\n");
		printf("Displaying application %s with %d arguments\n", Data.App[0], Data.NumArgs-1);
		for (int i=1; i<Data.NumArgs; i++)
			printf("Arg %d is %s\n", i, Data.App[i]);
		printf("When the gui is clicked the following buttons will do the following actions\n");
		printf("There are %d defined actions\n", Data.NumActions);
		for (int i=0; i<Data.NumActions; i++)
			printf("Action %d is %s\n", i, Data.Actions[i]);
		printf("Getting joystick data from shared memory file %s\n", Data.ButtonSeg);
		printf("Getting button data from shared memory file %s\n", Data.JoySeg);
	}
	//Return the data the program gathered
	return Data;
}
void WhenPressed()
{
	Data.Actions = new char*[10];
	Data.NumActions = 0;
	char* Token = new char[40];
	Token = strtok(NULL, " ");
	while (Token !=NULL)
	{
		Data.Actions[Data.NumActions] = strdup(Token);
		Data.NumActions++;
		Token = strtok(NULL, " ");

	}
	char* Read = Data.Actions[Data.NumActions-1];
	Read[strlen(Read) -1] = '\0';
}

void TriggerButton()
{
	char* Value = strtok(NULL, " ");
	Data.NumButtons = 0;
	Data.Buttons = new int[10];
	while (Value !=NULL)
	{
		Data.Buttons[Data.NumButtons] = atoi(Value);
		Data.NumButtons++;
		Value = strtok(NULL, " ");
	}
}
void StartApp()
{
	Data.App = new char*[10];
	Data.NumArgs = 0;
	char* Arg = strtok(NULL, " ");
	while (Arg !=NULL)
	{
		Data.App[Data.NumArgs] = strdup(Arg);
		Data.NumArgs++;
		Arg = strtok(NULL," ");
	}
	char* Read = Data.App[Data.NumArgs-1];
	Read[strlen(Read) -1] = '\0';
}

//XWand parsing function, handles getting the data out of the xwand configuration file
void XWandParser(char* File)
{
	FILE* InputFile;
	if (File == NULL)
		InputFile = fopen("xwand.config", "r");
	else
		InputFile = fopen(File, "r");
	char* Buff = new char[500];
	char* Token = new char[50];
	//Priming read
	fgets(Buff, 255, InputFile);
	while (!feof(InputFile))
	{
		Token = strtok(Buff, " ");
		switch(Token[0])
		{
			case '#':
			{
				break;
			}
			case 'W':
			{
				WhenPressed();
				break;
			}
			case 'P':
			{
				Data.Path = strdup(strtok(NULL, "\n"));
				break;
			}
			case 'T':
			{
				TriggerButton();
				break;
			}
			case 'I':
			{
				Data.InDir = atoi(strtok(NULL," "));
				break;
			}
			case 'S':
			{
				StartApp();
				break;
			}
			case 'D':
			{
				Data.Display = strdup(strtok(NULL, "\n"));
				break;
			}
			case 'J':
			{
				Data.JoySeg = strdup(strtok(NULL, "\n"));
				break;
			}
			case 'B':
			{
				Data.ButtonSeg = strdup(strtok(NULL, "\n"));
				break;
			}
			all:
			{
				printf("Unrecognized command in configuration file, please check your syntax\n");
				break;
			}
		}
		//Read in the next line
		fgets(Buff, 255, InputFile);
	}
}
