//
// DIVERSE graphics interface -- Performer ---- version 2.1.0
// released: Wed Jun 26 17:36:44 EDT 2002
// The DIVERSE HOMPAGE is at: www.diverse.vt.edu
// 
// Copyright (C) 2000, 2001, 2002  John Kelso, Lance Arsenault and Ron Kriz
// 
// This software, DIVERSE graphics interface -- Performer (DPF),
// is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License (LGPL) as
// published by the Free Software Foundation; either version 2.1 of
// the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this software, in the top level source directory in
// a file named "COPYING"; if not, see it at:
// http://www.gnu.org/copyleft/lesser.html or write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//
//
// DSO which displays text headsUp that updates each frame

#include <dpf.h>
#include "../CommandParser.h"
#include <dtk/dtkDSO_loader.h>
#include <sstream>
#define ROWS (36)
#define COLS (54)
#define START (ROWS-26)

class atomHeadsUp : public dpfHeadsUpText {
public:
  atomHeadsUp(dpf *app);
  int preConfig(void);
  int postFrame(void) ;

private:
  CommandParser * message;
  dpf *app ;
  int row, colorrow;

};


atomHeadsUp::atomHeadsUp(dpf *app_in):
  dpfHeadsUpText(app_in, "atomHeadsUp") {
  app = app_in ;
  setDescription("write text in headsUp window on first screen") ;
  message = new CommandParser("HEADSUP_MESSAGE");
  row = 0 ;
  colorrow = 0;
}

int atomHeadsUp::preConfig(void) {
  dpfDisplay *d = app->display();
  if(d == NULL) return DTK_CALLBACK_ERROR ;

  dpfPipe *p ;
  p = d->first();
  dpfPwin *w ;
  dpfScreen *s ;
//  for (p=d->first(); p; p=p->next()) {
  //  for (w=p->first(); w; w=w->next()) {
    //  for (s=w->first(); s; s=s->next()) {
	s = p->next()->first()->first();
	if (s){
	s->addHeadsUp(this) ;
	}


  if (init(ROWS, COLS)) {
     dtkMsg.add(DTKMSG_ERROR,
	       "atomHeadsUp::preConfig(): unable to init text\n") ;
    return DTK_CALLBACK_ERROR ;
  }

  screenPos(0, START+1, 0, COLS-1) ;
  visible(1);
  return DTK_REMOVE_CALLBACK ;
}

int atomHeadsUp::postFrame(void) {
  string ACommand;
  int returnvalue = message->getCommandLine(ACommand, "NONE");
  //printf("returnvalue=%u\n", returnvalue);
  istringstream parseLine(ACommand);
  string firstPart;
  getline(parseLine, firstPart, '\t');
  if (returnvalue==0)
  {
	if (ACommand == "clear"){
		for (int i=row-1; i >= 2; i--)
			sprintf(text(i,1),"%s", "                             ");
		for (int i=colorrow-1; i >= 2; i--)
			sprintf(text(i,30),"%s", "                            ");
		row = 2;
		colorrow = 2;
	}
	else if (firstPart == "color"){
		string theRest;
		getline(parseLine,theRest,'\n');
		sprintf(text(colorrow,30),"%s", theRest.c_str());
		colorrow++;
	}
	else {
		//printf("****%s\n", ACommand.c_str());
		sprintf(text(row,1),"%s", ACommand.c_str());
		row++;
	}
  }
  flush() ;

  return DTK_CONTINUE ;
}


/*************** DTK C++ DSO loader/unloader functions ***********/
/*
 * All DTK DSO files are required to declare these two functions.
 * These function are called by the loading program to get your
 * C++ objects loaded.
 *
 *****************************************************************/

static dtkAugment *dtkDSO_loader(dtkManager *manager, void *p)
{
  dpf *app = static_cast<dpf *>(manager);
  return new atomHeadsUp(app);
}

static int dtkDSO_unloader(dtkAugment *augment)
{
  delete augment;
  return DTKDSO_UNLOAD_CONTINUE;
}


