//------Standard C/C++ libraries
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <iostream>
#include <fstream>
#include <string>
using namespace std;
#include <sstream>
#include <vector>
//------John's Templates
#include "FlexArrayTmpl.H"

//------Header
#include "ReadAtomFiles.hh"

AtomType::AtomType(AtomType &at){
  for(int i=0;i<atoms.getSize();i++)
    getTypeNumber(at.atoms[i].name);
}
int AtomType::getTypeNumber(char *name){
  for(int i=0;i<atoms.getSize();i++)
    if(!strcmp(atoms[i].name,name))
      return i;
  //printf("Add new atomtype for %s\n",name);
  atoms.setSize(atoms.getSize()+1); // increase size
  //printf("AtomType size=%u\n",atoms.getSize());
  strcpy((atoms[atoms.getSize()-1]).name,name);
  return (atoms.getSize()-1);
}
AtomType &AtomType::operator=(AtomType &at){
  if(&at==this) return *this;
  //puts("Copy Atom Types... have %u types\n",at.nTypes());
  for(int i=0;i<at.nTypes();i++)
    getTypeNumber(at.atoms[i].name);
  return *this;
}

// copy constructor (for copying Atom Lists)
AtomList::AtomList(AtomList &a){
  atoms = a.atoms;
  bonds = a.bonds;
  textinfo = a.textinfo;
  molecule = a.molecule;
  atom_types = a.atom_types;
  Visible = a.Visible;
}

// copy operator (for copying Atom Lists)
AtomList &AtomList::operator=(AtomList &a){
  if(&a!=this){
    atoms=a.atoms;
    bonds = a.bonds;
    textinfo = a.textinfo;
    molecule=a.molecule;
    atom_types = a.atom_types;
    Visible = a.Visible;
  }
  return *this;
}
AtomList::AtomList(string filename){
  read(filename);
  //printf("Molecule %s\n", molecule.c_str());
}
void AtomList::read(string moleculename){
  ifstream inFile;
  string filename;
  string temp;
  string name;
  string DEF;
  string textstring;
  char cstringname[32];
  char ch;
  bool isAtom, isBond, isText;
  float x,y,z;
  unsigned int position;
  Visible = true;
  molecule = moleculename;
  istringstream parseMolecule(moleculename);
  getline(parseMolecule,filename,'_');
  if (filename == "mol")
  	getline(parseMolecule,filename,'_');
  filename = filename + ".x3d";
  //printf("Filename %s\n", filename.c_str());
  inFile.open(filename.c_str());
  ch = inFile.peek();
  //printf("Peek %s\n", &ch);
  while(inFile){
	isAtom = false;
	isBond = false;
	isText=false;
	getline(inFile,temp,'\n');
	if (temp.find("<ProtoInstance") != string::npos)
	{
		position=temp.find("name");
		if (position != string::npos)
		{
			istringstream parseLine(temp);
			parseLine.ignore(position);
			parseLine.ignore(256,'"');
			getline(parseLine,name,'"');
			if (name == "line")
				isBond = true;
		//	printf("Name %s\n", name.c_str());
		}
		position=temp.find("DEF");
		if (position != string::npos)
		{
			istringstream parseLine(temp);
			parseLine.ignore(position);
			parseLine.ignore(256,'"');
			getline(parseLine,DEF,'"');
		}
		position=temp.find("position");
		if (position != string::npos)
		{
			istringstream parseLine(temp);
			isAtom=true;
			parseLine.ignore(position);
			parseLine.ignore(256,'"');
			parseLine >> x >> y >> z;
		//	printf("X Y Z %f %f %f\n", x,y,z);
		}
		if (isAtom){
			AtomDef atom;
			atom.p.cart.x=x;
			atom.p.cart.y=y;
			atom.p.cart.z=z;
			atom.AtomName = DEF;
			strcpy(cstringname, name.c_str());
			atom.type = atom_types.getTypeNumber(cstringname);
			//printf("Appending %s\n",cstringname); // debug
			atoms.append(atom);
		}
		else if (isBond){
			//printf("Adding a bond");
			BondDef bond;
			getline(inFile,temp,'\n');
			istringstream parseLine(temp);
			position = temp.find("value");
			parseLine.ignore(position);
			parseLine.ignore(256,'"');
			parseLine >> x >> y >> z;
			//printf("X Y Z %f %f %f\n", x,y,z);
			bond.array[0] = x;
			bond.array[1] = y;
			bond.array[2] = z;
			parseLine.ignore(256,',');
			parseLine >> x >> y >> z;
			//printf("X Y Z %f %f %f\n", x,y,z);
			bond.array[3] = x;
			bond.array[4] = y;
			bond.array[5] = z;
			bond.BondName =  DEF;
			bonds.append(bond);

		}
	}
	else if (temp.find("<fieldValue") != string::npos)
	{
		position=temp.find("name");
		if (position != string::npos)
		{
			istringstream parseLine(temp);
			parseLine.ignore(position);
			parseLine.ignore(256,'"');
			getline(parseLine,name,'"');
			if (name == "txt")
				isText = true;
			//printf("Heads Up Text Name %s\n", name.c_str());
		}
		position=temp.find("value");
		if (position != string::npos)
		{
			istringstream parseLine(temp);
			parseLine.ignore(position);
			parseLine.ignore(256,'"');
			getline(parseLine,textstring,'"');
			//printf("Heads Up Text Value %s\n", textstring.c_str());


		}
		if (isText){
			textinfo.append(textstring);
		}
	}
  }
}

void AtomList::print(){
  printf("print() %s: \n", molecule.c_str());
  printf("Number of records = %u\n",atoms.getSize());
  for(int i=0;i<atoms.getSize();i++){
    printf("Name=%s: Type=%u: pos=%f:%f:%f\n",
           atoms[i].AtomName.c_str(),
	   atoms[i].type,
	   atoms[i].p.cart.x,atoms[i].p.cart.y,atoms[i].p.cart.z);
  }
}

MoleculeList::MoleculeList(){
	//atom_types = new AtomType [5];
}
// destructor (called when you "delete" this object)
MoleculeList::~MoleculeList(){ // nil for now
	//printf("Destroying molecule list");
}

bool MoleculeList::addMolecule(string molecule){
	int location = findMolecule(molecule);
	if (location == -1)
	{
		AtomList * newmolecule = new AtomList (molecule);
		//int size = molecules.getSize();
		printf("Pushing on new molecule\n");
		//printf("Pushing Molecule %s\n", newmolecule->molecule.c_str());
		molecules.append(*newmolecule);
		//printf("Number of types for newmolecule %u\n", newmolecule->atom_types.nTypes());
		//atom_types[size] = molecules[size].atom_types;
		return true;
	}
	else if (!molecules[location].Visible)
	{
		molecules[location].Visible = true;
	}
	return false;
}

bool MoleculeList::removeMolecule(string molecule){
	int location = findMolecule(molecule);
	if (location != -1)
	{
		printf("Removing molecule\n");
		molecules[location].Visible = false;
		return true;
	}
	return false;
}

int MoleculeList::findMolecule(string molecule){
	int arraysize = molecules.getSize();
	for (int i= 0; i < arraysize; i++)
        {
		//molecules[i].print();
		//printf("In Loop, index number=%u, current molecule= %s\n", i, molecules[i].molecule.c_str());
		//printf("In Loop, current molecule2= %s\n", molecule.c_str());
		if (molecule==molecules[i].molecule)
		{
			//printf("Molecule found");
			return i;
		}
	}
	//printf("Molecule not found");
	return -1;
}

void MoleculeList::findAtomID(string AtomName, int& moleculelocation, int& atomlocation){
	int arraysize = molecules.getSize();
	for (int i=0; i<arraysize;i++)
	{
		int numberatoms = molecules[i].atoms.getSize();
		for (int j=0;j<numberatoms; j++)
		{
			if (AtomName==molecules[i].atoms[j].AtomName)
			{
				moleculelocation = i;
				atomlocation = j;
				return;
			}
		}
	}
	moleculelocation = -1;
	atomlocation = -1;
}
void MoleculeList::findBondID(string BondName, int& moleculelocation, int& bondlocation){
	int arraysize = molecules.getSize();
	for (int i=0; i<arraysize;i++)
	{
		int numberbonds = molecules[i].bonds.getSize();
		for (int j=0;j<numberbonds; j++)
		{
			if (BondName==molecules[i].bonds[j].BondName)
			{
				moleculelocation = i;
				bondlocation = j;
				return;
			}
		}
	}
	moleculelocation = -1;
	bondlocation = -1;
}

void MoleculeList::getAtoms(string molecule,FlexArray<AtomDef> &atoms, FlexArray<BondDef> &bonds){
  int location = findMolecule(molecule);
  if (location != -1)
	{
		//printf("location %u", location);
		//printf("not at end");
		atoms=molecules[location].atoms;
		bonds=molecules[location].bonds;
	}
  //printf("getAtoms done");
}

void print(FlexArray<AtomDef> &atoms){
  printf("Number of records = %u\n",atoms.getSize());
  for(int i=0;i<10;i++){
    printf("Type=%u: pos=%f:%f:%f\n",
	   atoms[i].type,
	   atoms[i].p.cart.x,atoms[i].p.cart.y,atoms[i].p.cart.z);
  }
}
