#ifndef __READATOMFILES_HH_
#define __READATOMFILES_HH_
#include <string>
using namespace std;
// Convenience structures for locating a point in 3D space
struct sPoint { float x,y,z; };
union Point {
  float array[3];
  sPoint cart;
};

// Definition of an individual atom.
struct AtomDef {
  int type; // Integer type for the atom (generated from name by AtomType class)
  Point p;  // Location of the center of the atom.
  string AtomName;
};

struct BondDef {
  string BondName;
  float array[6];
};

// Brute-force name manager for atom namestrings
// It converts the name strings into index numbers
// and vice-verse.
class AtomType {
  // typedef char AtomTypeDefn[32];
  struct AtomTypeDefn {
    char name[32];
    // Default (empty) constructor.
    AtomTypeDefn(){}
    // Copy Constructor
    AtomTypeDefn(AtomTypeDefn &a){
      strcpy(name,a.name);
    }
    // Defines behavior of the '=' operation for assignment/
    AtomTypeDefn &operator=(AtomTypeDefn &a){
      if(&a==this) return *this;
      strcpy(name,a.name);
      return *this;
    }
  };
  FlexArray<AtomTypeDefn> atoms;
public:
  AtomType(){}
  AtomType(AtomType &at);
  // Converts a type name string into a unique integer number.
  // On first encounter with the string, it will create a new number for
  // it.  On subsequent encounters with the same string, it will
  // return the same number (uniquely identifying that string).
  int getTypeNumber(char *name);
  // retrieves the typename given the type number
  inline char *getTypeName(int typenumber){
    return (atoms[typenumber]).name;
  }
  // Gets the number of types currently identified (number of
  // uniquely identifiable strings).
  inline int nTypes(){ return atoms.getSize(); }
  // copy operator '='.
  AtomType &operator=(AtomType &at);
};

// A list of atoms (basically all of the atoms in the lattice at a particular
// timestep).
class AtomList {
public:
 bool Visible;
 string molecule;
  AtomType atom_types;
  FlexArray<AtomDef> atoms;
  FlexArray<BondDef> bonds;
  FlexArray<string> textinfo;
  AtomList(){}
  // copy constructor (for copying AtomDef Lists)
  AtomList(AtomList &a);
  AtomList(const AtomList &a);
  // copy operator (for copying AtomDef Lists using "=")
  AtomList &operator=(AtomList &a);
  AtomList(string filename);
  // Read the atom locations from an x,y,z file named <filename>
  void read(string filename);
  void print(); // debug print.
};

// Basically a time-sequence of AtomLists.  This linearly interpolates
// between the AtomLists to create smooth animations of the time-evolution
// of atom positions for the Cave application.
class MoleculeList {
public:
int findMolecule(string molecule);
void findBondID(string BondName, int& moleculelocation, int& bondlocation);
void findAtomID(string AtomName, int& moleculelocation, int& atomlocation);
FlexArray<AtomList> molecules;
  //AtomType * atom_types; // copied from 0th timestep (for convenience)
  MoleculeList();
  // destructor (called when you "delete" this object)
  ~MoleculeList();
  //Checks to see if the molecule exists and then adds it if it isn't already there
  bool addMolecule(string molecule);
  //Trys to find a molecule to remove
  bool removeMolecule(string molecule);
  // returns the Atoms for the specified molecule
  void getAtoms(string molecule,FlexArray<AtomDef> &atoms, FlexArray<BondDef> &bonds);
};

void print(FlexArray<AtomDef> &atoms); // Debug print statement

#endif
