#include <unistd.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <dpf.h>
#include <dtk.h>
#include <Performer/pf.h>
#include <Performer/pf/pfGeode.h>
#include <string>
#include <string.h>
#include "fstream"
using namespace std;
#include "AtomCommands.h"
#include "CommandParser.h"
#include "Messenger.h"
#include "FlexArrayTmpl.H"
#include "ReadAtomFiles.hh"
#include "SharedAtoms.h"
#include "colorhandler.h"
#include "atomview.h"

/*
//Set up the basicinformation necessary for displaying atoms
//This includes giving the program the base name of the file you are using
//the starting index, the ending index, the number of polygons you want to 
//display with, and a position to start at
//Your world is still centered at 0,0,0 but you are located at a different position
//that you specify.  For example, you want to appear right at the crack tip
//of a structure
int InitAtomview(char* FileName, AtomView& Program);
//This function basically takes a filename and tells the atomview program
//what color each element should be.  I.e. you can have Ni be blue and Al be red
//in your simulation or you can have them being yellow and purple
void SetColors(char* Filename, AtomView& Program, Messenger& Message);

//See above for documentation
//Filename = what file to open
//Program = the atomview program to use
//Message = the messenger class that communicates with atomview
int InitAtomview(char* FileName, AtomView& Program, Messenger& Message)
{
	printf("%s\n", FileName);
	FILE* InFile;
	InFile = fopen(FileName, "r");
	char* Buff = new char[800];
	//Open the file to get information out of it
	//ifstream In(FileName);
	//char *Buff = NULL;
	//Get the information out of it
	//if (!In)
	//{
	//	printf("Could not open file %s\n", FileName);
	//	return 1;
	//}
	///Read in the file config information
	//The base file information
	//string BaseName = ""; 
	//The information 
	char* BaseName = new char[100];
	double x=0,y=0,z=0,h=0,p=0,r=0;
	int Start=0,End=0,NumPoly=0;
	//Read the information
	fgets(Buff, 255, InFile);
	fgets(Buff, 255, InFile);
	sscanf(Buff, "%s %d %d\n", BaseName, &Start, &End);
	fgets(Buff, 255, InFile);
	fgets(Buff, 255, InFile);
	sscanf(Buff, "%d", &NumPoly);
	fgets(Buff, 255, InFile);
	fgets(Buff, 255, InFile);
	sscanf(Buff, "%lf %lf %lf %lf %lf %lf", &x,&y,&z,&h,&p,&r);
	printf("Got the # poly = %d\n", NumPoly);
	fclose(InFile);
	delete [] Buff;

	//Set up the file information and then send it over
	Buff = new char[500];
//Make a message
	sprintf(Buff, "%s\t%s\t%d\t%d", FILEINFO.c_str(), BaseName, Start, End);
	//Send the file information
	Message.SendCommand(string(Buff));
	Program.HandleMessages();
	delete [] BaseName;
	delete [] Buff; //Clean up memory
	//Set the number of polygons by default

	//Setup the polygon information and send it
	Buff = new char[50]; //Create a temporary buffer
	sprintf(Buff, "%s\t%d", NUMPOLYGONS.c_str(), NumPoly);
	Message.SendCommand(string(Buff)); //Send the message
	Program.HandleMessages(); //Handle the message
	delete [] Buff;

	//Setup the xyz hpr stuff and send it over
	Buff = new char[500];
	sprintf(Buff, "%s\t%lf\t%lf\t%lf\t%lf\t%lf\t%lf", SETORIGIN.c_str(), x,y,z,h,p,r);
	Message.SendCommand(string(Buff)); //Send the message
	Program.HandleMessages(); //Handle the message
	delete [] Buff;
	return 0;
}

//The parmaters are the same as for the above function
void SetColors(char* Filename, AtomView& Program, Messenger& Message)
{
	//Open file
	char* Buff = NULL; 
	FILE* InFile;
	InFile = fopen(Filename, "r");
	//Used to read from the file
	Buff = new char[800];
	fgets(Buff, 255,InFile );
	char* AtomName  = new char[50];
	char* AtomColor = new char[50];
	//Priming read
	sscanf(Buff, "%s %s", AtomName, AtomColor);
	//Until end of file create a message, send it off, process it, and then read in the next atom / color combo
	while (!feof(InFile))
	{
		sprintf(Buff, "%s\t%s\t%s", ATOMCOLOR.c_str(), AtomName, AtomColor);
		Message.SendCommand(string(Buff));
		Program.HandleMessages();
		fgets(Buff, 255,InFile );
		sscanf(Buff, "%s %s", AtomName, AtomColor);
	}
	//Cleanup
	fclose(InFile);
	delete [] Buff;
	delete [] AtomName;
	delete [] AtomColor;
	printf("Finished with colors :)\n");
}
*/

//Basically setup the information for the atomview program and then start the display for it
int main(int argc, char** argv, char** env)
{
	/*if (argc <2 || argc >3)
	{
		printf("Usage is %s displayinfofile colorinfofile(optional)\n", argv[0]);
		return 1;
	}*/
	printf("Program Started");
	AtomView TheProgram(env);
	Messenger Message("ATOM_MESSAGE");
	//Prime the message system
	Message.SendCommand(NOTHING);
	TheProgram.HandleMessages();
	printf("Message Handler primed");
	
	TheProgram.StartDisplay();
	return 0;
}
