#ifndef atomview_h
#define atomview_h
#include <string>
#include "SysTimer.h"
using namespace std;

//Structure with the name of the element, its color, and the geode that it points to
//This is for ease of data handling
struct Colors
{
	//The name of the atom
	char* Name;
	//The geode that it points to
	pfNode* ColorNode;
};
//Used to map atom name to atomcolor
struct ColorMap
{
	char* Name;
	char* Color;
};

struct AtomConstants{
	string Name;
	float AtomicRadius;
};

class MoleculeDisplay
{
	public:
	MoleculeDisplay(const MoleculeDisplay &a);
  	// copy operator (for copying AtomDef Lists using "=")
  	MoleculeDisplay &operator=(MoleculeDisplay &a);
	//The environmental variables
	MoleculeDisplay();
	MoleculeDisplay(string LoadMe);
	~MoleculeDisplay();
	string MoleculeName;
	int NumColors;
	float center[3];
	pfDCS* MolScaler;
	pfDCS** BondDisplay;
	pfDCS** Display;
	//This is the number of atoms stored in the current tree
	int NumAtoms, NumBonds;
	//This is a listing of the atom names so I can keep track
	//of whether or not an atom actually changed
	char** AtomNames;
	//The number of names in the atom name  list
	int NumAtomNames;
	int Order;
	char* findColorByName(char* name);
	//The map between an atom and its color
	ColorMap* CMap;
	//This holds the color information for each atom type
	Colors* ColorData; // Name and Corresponding node
	//GeoColor is the mapping of a color to a geode
	//This simplifies displaying objects immensley
	ColorHandler* GeoColor; // Name and color

};

//This is the real workhorse behind the atomview program
//You basically create one of these
class AtomView
{
	public:
	//Init atomview
	AtomView(char** Env);
	//Destroy atomview
	~AtomView();
	//Start the performer display
	void StartDisplay();
	void highlightAtom(string AtomName);
	void highlightBond(string BondName);
	float getAtomRadius(string atomname);
	float getCurrentTrans(int Cur);
	void CheckPosition();
	//Handle messages sent to the program
	string HandleMessages();
	private:
	SysTimer myTimer;
	// Read the colors into the Molecule Display
	void setColors(string Filename);
	//Actually start the performer data loop
	int RealDisplay();
	void dehighlightAll();
	//Do some setup before doing the real gruntwork
	void InitDisplay();
	//Create lighting for the program
	void MakeLighting();
	//Map colors to each atom
	void MapColors();
	//Make the tree of atoms to display
	void MakeTree();
	//Update the tree of atoms
	//void UpdateTree();
	//Get an array of the atom names
	void MapNames();
	//Are we supposed to go away?
	bool Die;
	//Implements the teeth behind the messages
	void DoCommand(const string& ACommand);
	//The wrapper class for atoms, can be abstracted to hold any
	//data though
	DTKSharedAtoms* AtomList;
	//The message parser
	CommandParser* message;
	//Send messages back to myself or more importantly , other processes running around waiting on info
	Messenger* SelfMessage;
	//Temporary atomview data objects used to hold the
	//current set of atoms and for use to be written to  by the master data handler(AtomList)
	FlexArray<AtomDef> New;
	FlexArray<BondDef> ArrayBondDefs;
	//The number of polygons per sphere
	int NumPolygons;
	//The scale for each sphere
	double TheScale;
	//The scale for the world
	double WorldScale;
	bool ReadStuff;
	AtomConstants * CMLConstants;
	//The size of the world before the change
	double OldWorldScale;
	string MoleculeName;
	string AtomName;
	string BondName;
	//New molecule needs to be loaded
	bool Load;
	//Molecule needs to be unloaded
	bool Unload;
	//Atom needs to be highlighted
	bool Atom;
	//Bond needs to be highlighted
	bool Bond;
	//Has the world scale changed
	bool WorldScaleChange;
	//If the world is larger set this (for scaling)
	bool WorldLarger;
	//If the world is smaller set this (for scaling)
	bool WorldSmaller;
	//Has the scale for each sphere changed?
	bool ScaleChange;
	//Are we scaling on the origin
	bool OriginScale;
	//Are we scaling on the position
	bool PosScale;
	//For catching singals in the display
	//The dpf object that holds everything, it is set in the RealDisplay function
	dpf *App;
	dpfDisplay * d;
	//This is  used to tell if the atoms have changed
	bool AtomsChanged;
	//The origin cooridantes
	double x,y,z,h,p,r;
	float curpos[6];
	int PreviousID;
	//Has the origin changed?
	bool OriginChange;
	//GeoColor is the mapping of a color to a geode
	//This simplifies displaying objects immensley
	//ColorHandler* GeoColor;
	//The environmental variables
	char** Environ;
	//The map between an atom and its color
	//ColorMap* CMap;
	//The top node in the tree
	pfDCS* Scaler;
	//Array that holds all the information needed to display molecules
	FlexArray<MoleculeDisplay> MolDArray;
	int CurMolID;
};
//Legacy , left in just in case someone wants to make it work so that performers bad
//signal handling can be replaced
void signal_catcher(const int& sig);
#endif
