#include <unistd.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <dpf.h>
#include <dtk.h>
#include <Performer/pf.h>
#include <Performer/pr.h>
#include <Performer/pf/pfGeode.h>
#include <Performer/pr/pfMaterial.h>
#include <string>
using namespace std;
#include "FlexArrayTmpl.H"
#include "ReadAtomFiles.hh"
#include "SharedAtoms.h"
#include "CommandParser.h"
#include "Messenger.h"
#include "AtomCommands.h"
#include "colorhandler.h"
#include "atomview.h"

void AtomView::MapColors()
{
	//We only support 7 types now
	MolDArray[CurMolID].NumColors = 16;


	int NumTypes = AtomList->FileData->molecules[CurMolID].atom_types.nTypes();
	//printf("NumTypes= %u", NumTypes);
	if (NumTypes > 16)
		NumTypes = 16;
	MolDArray[CurMolID].NumColors = NumTypes;
	//Go through and get the names
	//Set the names both in the color data and in the color map
	for (int i=0; i< NumTypes; i++)
		strcpy(MolDArray[CurMolID].ColorData[i].Name ,AtomList->FileData->molecules[CurMolID].atom_types.getTypeName(i));
	for (int i=NumTypes; i<16; i++)
		strcpy(MolDArray[CurMolID].ColorData[i].Name, "--");
//	for (int i=0; i<NumTypes; i++)
//		printf("%s has color %s\n", ColorData[i].Name, CMap[i].Color);
	//Associate the right geode with the right color
	for (int i=0; i<MolDArray[CurMolID].NumColors; i++)
	{
		for (int j=0; j<MolDArray[CurMolID].NumColors; j++)
		{
			if (!strcmp(MolDArray[CurMolID].ColorData[i].Name, MolDArray[CurMolID].CMap[j].Name))
				MolDArray[CurMolID].ColorData[i].ColorNode = MolDArray[CurMolID].GeoColor->GetColor(MolDArray[CurMolID].CMap[j].Color);
		}
	}
	//for (int i=0; i<16; i++)
	//	printf("%s %s %s\n", MolDArray[CurMolID].CMap[i].Name, MolDArray[CurMolID].CMap[i].Color, MolDArray[CurMolID].ColorData[i].Name);
//	printf("I have %d types\n", NumTypes);
//printf("Exit NumTypes= %u", NumTypes);
}

//This function is called when a new molecule is read in
void AtomView::setColors(string Filename)
{
	//Open file
	Messenger Message ("ATOM_MESSAGE");


	char* Buff = NULL;
	FILE* InFile;
	string filename;
	istringstream parseFilename(Filename);
  	getline(parseFilename,filename,'_');
  	if (filename == "mol")
  		getline(parseFilename,filename,'_');
	filename = filename + ".color";
	InFile = fopen(filename.c_str(), "r");
	//Used to read from the file
	Buff = new char[800];
	fgets(Buff, 255,InFile );
	char* CAtomName  = new char[50];
	char* CAtomColor = new char[50];
	//Priming read
	sscanf(Buff, "%s %s", CAtomName, CAtomColor);
	//Until end of file create a message, send it off, process it, and then read in the next atom / color combo
	while (!feof(InFile))
	{
		//printf("%s\t%s\t%s", ATOMCOLOR.c_str(), CAtomName, CAtomColor);
		sprintf(Buff, "%s\t%s\t%s", ATOMCOLOR.c_str(),CAtomName, CAtomColor);


		string AtomName  = CAtomName;
		string AtomColor = CAtomColor;

			for (int i=0; i<16; i++)
			{
				if (!strcmp(MolDArray[CurMolID].CMap[i].Name, "--"))
				{
					strcpy(MolDArray[CurMolID].CMap[i].Color , AtomColor.c_str());
					strcpy(MolDArray[CurMolID].CMap[i].Name  , AtomName.c_str());
					break;
				}
			}
		fgets(Buff, 255,InFile );
		sscanf(Buff, "%s %s", CAtomName, CAtomColor);
	}
	//Cleanup
	fclose(InFile);
	delete [] Buff;
	delete [] CAtomName;
	delete [] CAtomColor;
	//printf("Finished with colors :)\n");
}


char* MoleculeDisplay::findColorByName(char* name){
	for (int i=0; i < NumColors; i++){
		//printf("CMap Name=%s, Color=%s\n", CMap[i].Name, CMap[i].Color);
		if (!strcmp(CMap[i].Name, name))
			return CMap[i].Color;
	}
	return ("not found");
}

float AtomView::getAtomRadius(string atomname){
	for (int i=0; i <11; i++){
		if (CMLConstants[i].Name == atomname)
			return CMLConstants[i].AtomicRadius;
	}
	return -1.0;

}

void AtomView::MakeTree()
{
	//Get the correct number of atoms and bonds to display
	MolDArray[CurMolID].NumAtoms = New.getSize();
	MolDArray[CurMolID].NumBonds = ArrayBondDefs.getSize();
	//printf("Current molid %u\n", CurMolID);


	//Setup the array to be used to keep track of atom names

	//MolDArray[CurMolID].MaxAtoms = MolDArray[CurMolID].NumAtoms+1000;
	MolDArray[CurMolID].AtomNames = new char*[MolDArray[CurMolID].NumAtoms];
	for (int i=0; i< MolDArray[CurMolID].NumAtoms; i++)
	{
		MolDArray[CurMolID].AtomNames[i] = new char[2];
		strcpy(MolDArray[CurMolID].AtomNames[i], "--");
	}

	MolDArray[CurMolID].NumAtomNames = MolDArray[CurMolID].NumAtoms;
	//Correctly map atoms to their respective colors
	for (int i=0; i<16; i++)
	{
		if (MolDArray[CurMolID].ColorData[i].Name != NULL)
			strcpy(MolDArray[CurMolID].ColorData[i].Name, "--");
	}

	//printf("Make the geodes\n");
	if (MolDArray[CurMolID].GeoColor == NULL)
		MolDArray[CurMolID].GeoColor = new ColorHandler(NumPolygons);
	//Stuff the default colors into the color map
	for (int i=0; i<16; i++)
		MolDArray[CurMolID].ColorData[i].ColorNode = MolDArray[CurMolID].GeoColor->GetColor(MolDArray[CurMolID].CMap[i].Color);


	//printf("Before mapping colors\n");
	MapColors();
	//printf("After mapping colors\n");

	MapNames();
	//Create the scale node


	//Make a bunch of node pointers to hold the atoms
	//Display = new pfDCS*[MaxAtoms];
	MolDArray[CurMolID].Display = (pfDCS**)malloc(sizeof(pfDCS*)*MolDArray[CurMolID].NumAtoms);
	for (int i=0; i< MolDArray[CurMolID].NumAtoms ; i++)
	{
		MolDArray[CurMolID].Display[i] = new pfDCS();
		//Display[i] = (pfDCS*)malloc(sizeof(pfDCS));
	}
	//Set the color of the each atom
	//Colordata has at most 7 entries so I basically just
	//access which color to put in by going to what type it
	//is and getting the color from that
	float AtomicRadius;
	for (int i=0; i<MolDArray[CurMolID].NumAtoms; i++)
	{
		MolDArray[CurMolID].Display[i]->addChild(MolDArray[CurMolID].ColorData[New[i].type].ColorNode);
		MolDArray[CurMolID].Display[i]->setTrans(New[i].p.cart.x, New[i].p.cart.y, New[i].p.cart.z);
		AtomicRadius = getAtomRadius(MolDArray[CurMolID].ColorData[New[i].type].Name);
		//printf("AtomicRadius = %f\n", AtomicRadius);
		MolDArray[CurMolID].Display[i]->setScale(TheScale * AtomicRadius);
	}

	MolDArray[CurMolID].BondDisplay = (pfDCS**)malloc(sizeof(pfDCS*)*MolDArray[CurMolID].NumBonds);
	for (int i=0; i< MolDArray[CurMolID].NumBonds ; i++)
	{
		MolDArray[CurMolID].BondDisplay[i] = new pfDCS();
		//Display[i] = (pfDCS*)malloc(sizeof(pfDCS));
	}
	//Set the geode for the bonds and add to screen
	for (int i=0; i<MolDArray[CurMolID].NumBonds; i++)
	{
		//Hard code in correct geode. Not great, but it works
		MolDArray[CurMolID].BondDisplay[i]->addChild(MolDArray[CurMolID].ColorData[7].ColorNode);
		double pi = 3.1415926535;
		double r2d = 180.0/pi;	// radians to degrees conversion factor


		// orientation vector
		double vx = ArrayBondDefs[i].array[3]-ArrayBondDefs[i].array[0];	//	component in x-direction
		double vy = ArrayBondDefs[i].array[4]-ArrayBondDefs[i].array[1];	//	component in y-direction
		double vz = ArrayBondDefs[i].array[5]-ArrayBondDefs[i].array[2];	//	component in z-direction

		double midx = (ArrayBondDefs[i].array[3]+ArrayBondDefs[i].array[0])/2;
		double midy = (ArrayBondDefs[i].array[4]+ArrayBondDefs[i].array[1])/2;
		double midz = (ArrayBondDefs[i].array[5]+ArrayBondDefs[i].array[2])/2;

		double v = sqrt( vx*vx + vy*vy + vz*vz );	// cylinder length
		//printf("*****************\n cylinder length = %f,%f,%f,%f", v, midx,midy,midz);
		// rotation vector, z x r
		double rx = -vy*vz;
		double ry = +vx*vz;
		double ax = 0.0;

		double zero = 1.0e-3;
		if (fabs(vz) < zero)
		{
			ax = r2d*acos( vx/v );	// rotation angle in x-y plane
			if ( vx <= 0.0 ) ax = -ax;
		}
		else
		{
			ax = r2d*acos( vz/v );	// rotation angle
			if ( vz <= 0.0 ) ax = -ax;
		}

		pfMatrix coordMat,rotMat;
		coordMat.makeIdent();
		if (fabs(vz) < zero)
		{
			rotMat.makeRot(90.0, 0, 1, 0.0);			// Rotate & align with x axis
			rotMat.postMult(coordMat);
			rotMat.makeRot(ax, -1.0, 0.0, 0.0);		// Rotate to point 2 in x-y plane
			rotMat.postMult(coordMat);
		}
		else
		{
			rotMat.makeRot(ax, rx, ry, 0.0);			// Rotate about rotation vector
			rotMat.postMult(coordMat);
		}
		rotMat.preScale(0.01f,0.01f,v*.5, rotMat);
		MolDArray[CurMolID].BondDisplay[i]->setMat(rotMat);
		MolDArray[CurMolID].BondDisplay[i]->setTrans(midx, midy, midz);
	}
	//This is the node underneath the world that allows me to scale the world easily
	if (Scaler ==NULL)
	{
		Scaler = new pfDCS;

		//Set the scale of the node to be what we want
		//Scaler->setScale(TheScale);
	}
	//printf("Before adding to MolScaler\n");

	if (MolDArray[CurMolID].MolScaler ==NULL)
	{
		//printf("Scaler Created\n");
		MolDArray[CurMolID].MolScaler = new pfDCS();

		//MolDArray[CurMolID].MolScaler->setScale(TheScale);
	}
	//printf("After adding to MolScaler\n");

	//Go through and set up the atoms at their correct x,y,z points and add them below the scale node
	for (int i=0; i< MolDArray[CurMolID].NumAtoms; i++)
	{

		MolDArray[CurMolID].MolScaler->addChild(MolDArray[CurMolID].Display[i]);

	}
	//printf("MolScaler Displays added\n");
	for (int i=0; i< MolDArray[CurMolID].NumBonds; i++)
	{
		//MolDArray[CurMolID].BondDisplay[i]->setScale(TheScale);
		MolDArray[CurMolID].MolScaler->addChild(MolDArray[CurMolID].BondDisplay[i]);

	}
	//printf("MolDArray size %u\n", MolDArray.getSize());
	pfSphere tempSphere;
	MolDArray[CurMolID].MolScaler->getBound(&tempSphere);
	MolDArray[CurMolID].center[0] = tempSphere.center[0];
	MolDArray[CurMolID].center[1] = tempSphere.center[1];
	MolDArray[CurMolID].center[2] = tempSphere.center[2];
	float nodetrans = getCurrentTrans(CurMolID);
	//printf("Nodetrans=%f.\n", nodetrans);
	MolDArray[CurMolID].MolScaler->setTrans(nodetrans, 0, 0);
	//MolDArray[CurMolID].MolScaler->setScale(TheScale);
	Scaler->addChild(MolDArray[CurMolID].MolScaler);
	//Add all the other nodes to the tree but dont' show anything
	//for (int i=MolDArray[CurMolID].NumAtoms; i<MolDArray[CurMolID].MaxAtoms; i++)
	//	Scaler->addChild(MolDArray[CurMolID].Display[i]);
	//printf("MakeTree done");
}

void AtomView::MapNames()
{
	int NumTypes = AtomList->FileData->molecules[(int)AtomList->Curr()].atom_types.nTypes();
	for (int i=0; i<MolDArray[CurMolID].NumAtoms; i++)
	{
		/*
		printf("Type is= %d\n",New[i].type);

		printf("Name1 = %s\n",AtomNames[i]);
		printf("Name2 = %s\n",ColorData[New[i].type].Name);
		printf("Numatoms = %d and Real size is %d\n", NumAtoms, New.getSize());
		printf("At position = %d %d \n", i,  NumTypes);
		*/
		strcpy(MolDArray[CurMolID].AtomNames[i], MolDArray[CurMolID].ColorData[New[i].type].Name);
	}
	MolDArray[CurMolID].NumAtomNames = MolDArray[CurMolID].NumAtoms;
}
