#include <unistd.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <dpf.h>
#include <dtk.h>
#include <Performer/pf.h>
#include <Performer/pf/pfGeode.h>
#include <string>
#include <fstream>
using namespace std;
#include "AtomCommands.h"
#include "FlexArrayTmpl.H"
#include "ReadAtomFiles.hh"
#include "SharedAtoms.h"
#include "CommandParser.h"
#include "Messenger.h"
#include "colorhandler.h"
#include "atomview.h"

MoleculeDisplay::MoleculeDisplay(){
	//printf("Stupid Constructor");
}

// copy constructor (for copying Atom Lists)
MoleculeDisplay::MoleculeDisplay(const MoleculeDisplay &a){
	//printf("Molecule Display Constructor");
	MoleculeName = a.MoleculeName;
	NumColors = a.NumColors;
 	//ColorData = new Colors[NumColors];
	ColorData = a.ColorData;
	Order = a.Order;
	center[0] = a.center[0];
	center[1] = a.center[1];
	center[2] = a.center[2];
	/*for (int i=0; i<NumColors; i++)
	{
		ColorData[i].Name = new char[2];
		strcpy(ColorData[i].Name, a.ColorData[i].Name);
		ColorData[i].ColorNode =  a.ColorData[i].ColorNode;
	}*/
	GeoColor=a.GeoColor;//fix
	Display = a.Display;//fix
	BondDisplay = a.BondDisplay;
	MolScaler = a.MolScaler;
	CMap = a.CMap;
	/*CMap = new ColorMap[NumColors];
	for (int i=0; i<NumColors; i++)
	{
		CMap[i].Name = new char[2];
		strcpy(CMap[i].Name, a.CMap[i].Name);
		CMap[i].Color = new char[40];
		strcpy(CMap[i].Color, a.CMap[i].Color);
	}*/
	NumAtoms = a.NumAtoms;
	NumBonds = a.NumBonds;
	AtomNames =a.AtomNames;//fix
	NumAtomNames =a.NumAtomNames;
}

// copy operator (for copying Atom Lists)
MoleculeDisplay &MoleculeDisplay::operator=(MoleculeDisplay &a){
  if(&a!=this)
    {
	//printf("Molecule Display Copy Operator");
	MoleculeName = a.MoleculeName;
	NumColors = a.NumColors;
	ColorData = a.ColorData;
	Order = a.Order;
	//ColorData = new Colors[NumColors];
	center[0] = a.center[0];
	center[1] = a.center[1];
	center[2] = a.center[2];
	/*for (int i=0; i<NumColors; i++)
	{
		ColorData[i].Name = new char[2];
		strcpy(ColorData[i].Name, a.ColorData[i].Name);
		ColorData[i].ColorNode = a.ColorData[i].ColorNode;
	}*/
	GeoColor=a.GeoColor;//fix
	Display = a.Display;//fix
	BondDisplay = a.BondDisplay;
	MolScaler = a.MolScaler;
	CMap = a.CMap;
	/*CMap = new ColorMap[NumColors];
	for (int i=0; i<NumColors; i++)
	{
		CMap[i].Name = new char[2];
		strcpy(CMap[i].Name, a.CMap[i].Name);
		CMap[i].Color = new char[40];
		strcpy(CMap[i].Color, a.CMap[i].Color);
	}*/
	NumAtoms = a.NumAtoms;
	NumBonds = a.NumBonds;
	AtomNames =a.AtomNames;//fix
	NumAtomNames =a.NumAtomNames;
    }

  return *this;
}

MoleculeDisplay::MoleculeDisplay(string LoadMe){
	//printf("Constructing molecule");
	MoleculeName = LoadMe;
	NumColors = 16;
	Order = -1;
	ColorData = new Colors[NumColors];
	//BeingDisplayed = true;
	for (int i=0; i<NumColors; i++)
	{
		ColorData[i].Name = new char[2];
		strcpy(ColorData[i].Name, "--");
		ColorData[i].ColorNode = NULL;
	}
	GeoColor=NULL;
	Display = NULL;
	BondDisplay = NULL;
	MolScaler = NULL;
	CMap = new ColorMap[NumColors];
	for (int i=0; i<NumColors; i++)
	{
		CMap[i].Name = new char[2];
		strcpy(CMap[i].Name, "--");
		CMap[i].Color = new char[40];
	}
	//Setup the default colors for everything
	strcpy(CMap[0].Color, "blue");
	strcpy(CMap[1].Color, "red");
	strcpy(CMap[2].Color, "green");
	strcpy(CMap[3].Color, "yellow");
	strcpy(CMap[4].Color, "orange");
	strcpy(CMap[5].Color, "white");
	strcpy(CMap[6].Color, "purple");
	strcpy(CMap[7].Color, "bondcolor");
	strcpy(CMap[8].Color, "bluemesh");
	strcpy(CMap[9].Color, "redmesh");
	strcpy(CMap[10].Color, "greenmesh");
	strcpy(CMap[11].Color, "yellowmesh");
	strcpy(CMap[12].Color, "orangemesh");
	strcpy(CMap[13].Color, "whitemesh");
	strcpy(CMap[14].Color, "purplemesh");
	strcpy(CMap[15].Color, "bondcolormesh");
	NumAtoms =0;
	NumBonds =0;
	AtomNames =NULL;
	NumAtomNames = 0;
}

MoleculeDisplay::~MoleculeDisplay()
{
	/*printf("Destroying the MoleculeDisplay\n");
	if (GeoColor !=NULL)
	{
		printf("Destroying GeoColor\n");
		delete GeoColor;
	}
	for (int i=0; i<NumColors; i++)
	{
		if (ColorData[i].Name != NULL)
			delete [] ColorData[i].Name;
	}*/
}

//Initalize variables
AtomView::AtomView(char** Env)
{
	Environ=Env;
	Load = false;
	Unload = false;
	Atom = false;
	Bond = false;
	Die = false;
	AtomList = NULL;
	message = NULL;
	SelfMessage = NULL;
	AtomsChanged = false;
	TheScale = .2;
	WorldScale = 4;
	OldWorldScale = 4;
	WorldScaleChange = false;
	ScaleChange = false;
	WorldSmaller = false;
	WorldLarger = false;
	NumPolygons = 100;
	OriginScale = false;
	PosScale = false;
	ReadStuff = false;
	//Start position info
	x=0;
	y=0;
	z=0;
	h=0;
	p=0;
	r=0;
	PreviousID = -1;
	OriginChange = false;
	Scaler = NULL;
	CMLConstants = new AtomConstants [11];
	int i=0;
	std::ifstream inFile;
	inFile.open("cmlconstants.txt");
	inFile.ignore(500, '\n');
	while (inFile && i < 11){
		inFile.ignore(500, '\t');
		inFile.ignore(500, '\t');
		inFile >> CMLConstants[i].Name;
		printf("CMLConstant name %s\n", CMLConstants[i].Name.c_str());
		inFile.ignore(500, '\t');
		inFile.ignore(500, '\t');
		inFile >> CMLConstants[i].AtomicRadius;
		printf("CMLConstant radius %f\n", CMLConstants[i].AtomicRadius);
		inFile.ignore(500, '\n');
		i++;
	}
}


//Cleanup, yes some programmers know how to make thier code cleanup after itself, or attempt to make it seem like they are!
AtomView::~AtomView()
{
	printf("Destroying the atomview data program.  Thank you for using Atomview !!!!!!!!!!!!!!!!\n");
	if (SelfMessage != NULL)
		delete SelfMessage;
	if (AtomList != NULL)
		delete AtomList;
	delete AtomList;
}
