#include <unistd.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <dpf.h>
#include <dtk.h>
#include <Performer/pf.h>
#include <Performer/pr.h>
#include <Performer/pf/pfGeode.h>
#include <Performer/pr/pfMaterial.h>
#include <string>
using namespace std;
#include "FlexArrayTmpl.H"
#include "ReadAtomFiles.hh"
#include "SharedAtoms.h"
#include "CommandParser.h"
#include "Messenger.h"
#include "AtomCommands.h"
#include "colorhandler.h"
#include "atomview.h"
void AtomView::highlightBond(string BondName){
	int mollocation;
	int realid;
	printf("Highlight Bond\n");
	AtomList->FileData->findBondID(BondName, mollocation, realid);
	//printf("mollocation %u, realid %u", mollocation, realid);
	if (mollocation != -1)
	{
		CurMolID = mollocation;
		//printf("Number of Childeren %u", MolDArray[CurMolID].BondDisplay[realid]->getNumChildren());
		if (MolDArray[CurMolID].BondDisplay[realid]->searchChild(MolDArray[CurMolID].ColorData[7].ColorNode)!=-1){
			MolDArray[CurMolID].MolScaler->removeChild(MolDArray[CurMolID].BondDisplay[realid]);
			AtomList->GetAtoms(MolDArray[CurMolID].MoleculeName);
			printf("Putting highlight color in\n");
			MolDArray[CurMolID].BondDisplay[realid]->removeChild(MolDArray[CurMolID].BondDisplay[realid]->getChild(0));
			MolDArray[CurMolID].BondDisplay[realid]->addChild(MolDArray[CurMolID].ColorData[15].ColorNode);


			double pi = 3.1415926535;
			double r2d = 180.0/pi;	// radians to degrees conversion factor


			// orientation vector
			double vx = ArrayBondDefs[realid].array[3]-ArrayBondDefs[realid].array[0];	//	component in x-direction
			double vy = ArrayBondDefs[realid].array[4]-ArrayBondDefs[realid].array[1];	//	component in y-direction
			double vz = ArrayBondDefs[realid].array[5]-ArrayBondDefs[realid].array[2];	//	component in z-direction

			double midx = (ArrayBondDefs[realid].array[3]+ArrayBondDefs[realid].array[0])/2;
			double midy = (ArrayBondDefs[realid].array[4]+ArrayBondDefs[realid].array[1])/2;
			double midz = (ArrayBondDefs[realid].array[5]+ArrayBondDefs[realid].array[2])/2;

			double v = sqrt( vx*vx + vy*vy + vz*vz );	// cylinder length
			//printf("*****************\n cylinder length = %f,%f,%f,%f", v, midx,midy,midz);
			// rotation vector, z x r
			double rx = -vy*vz;
			double ry = +vx*vz;
			double ax = 0.0;

			double zero = 1.0e-3;
			if (fabs(vz) < zero)
			{
				ax = r2d*acos( vx/v );	// rotation angle in x-y plane
				if ( vx <= 0.0 ) ax = -ax;
			}
			else
			{
				ax = r2d*acos( vz/v );	// rotation angle
				if ( vz <= 0.0 ) ax = -ax;
			}

			pfMatrix coordMat,rotMat;
			coordMat.makeIdent();
			if (fabs(vz) < zero)
			{
				rotMat.makeRot(90.0, 0, 1, 0.0);			// Rotate & align with x axis
				rotMat.postMult(coordMat);
				rotMat.makeRot(ax, -1.0, 0.0, 0.0);		// Rotate to point 2 in x-y plane
				rotMat.postMult(coordMat);
			}
			else
			{
				rotMat.makeRot(ax, rx, ry, 0.0);			// Rotate about rotation vector
				rotMat.postMult(coordMat);
			}
			rotMat.preScale(0.02f,0.02f,v*.5, rotMat);
			MolDArray[CurMolID].BondDisplay[realid]->setMat(rotMat);
			MolDArray[CurMolID].BondDisplay[realid]->setTrans(midx, midy, midz);
			//MolDArray[CurMolID].Display[realid]->setTrans(New[realid].p.cart.x, New[realid].p.cart.y, New[realid].p.cart.z);
			//printf("%f,%f,%f,",New[realid].p.cart.x, New[realid].p.cart.y, New[realid].p.cart.z);
			MolDArray[CurMolID].MolScaler->addChild(MolDArray[CurMolID].BondDisplay[realid]);
		}
		else
		{
			printf("Putting regular color back\n");
			MolDArray[CurMolID].MolScaler->removeChild(MolDArray[CurMolID].BondDisplay[realid]);
			MolDArray[CurMolID].BondDisplay[realid]->removeChild(MolDArray[CurMolID].BondDisplay[realid]->getChild(0));
			MolDArray[CurMolID].BondDisplay[realid]->addChild(MolDArray[CurMolID].ColorData[7].ColorNode);
			MolDArray[CurMolID].MolScaler->addChild(MolDArray[CurMolID].BondDisplay[realid]);
		}
	}
	else
	{
		printf("Not highlighting");
	}
}

void AtomView::highlightAtom(string AtomName){
	int mollocation;
	int realid;
	//printf("Highlight Atom\n");
	AtomList->FileData->findAtomID(AtomName, mollocation, realid);

	if (mollocation != -1)
	{
		CurMolID = mollocation;
		AtomList->GetAtoms(MolDArray[CurMolID].MoleculeName);
		string curcolor = MolDArray[CurMolID].findColorByName(AtomList->FileData->molecules[CurMolID].atom_types.getTypeName(New[realid].type));
		if (MolDArray[CurMolID].Display[realid]->searchChild(MolDArray[CurMolID].GeoColor->GetColor(curcolor.c_str()))!=-1){
			MolDArray[CurMolID].MolScaler->removeChild(MolDArray[CurMolID].Display[realid]);
			AtomList->GetAtoms(MolDArray[CurMolID].MoleculeName);
			//printf("Atom Info %f,%f,%f,%u",New[realid].p.cart.x, New[realid].p.cart.y, New[realid].p.cart.z, New[realid].type);
			//printf("Current Atom = %s, Color = %s", AtomList->FileData->molecules[CurMolID].atom_types.getTypeName(New[realid].type),curcolor.c_str());
			//MolDArray[CurMolID].Display[realid]->removeChild(MolDArray[CurMolID].Display[realid]->getChild(1));
			MolDArray[CurMolID].Display[realid]->setTrans(New[realid].p.cart.x, New[realid].p.cart.y, New[realid].p.cart.z);
			MolDArray[CurMolID].Display[realid]->removeChild(MolDArray[CurMolID].Display[realid]->getChild(0));
			curcolor = curcolor + "mesh";
			MolDArray[CurMolID].Display[realid]->addChild(MolDArray[CurMolID].GeoColor->GetColor(curcolor.c_str()));
			MolDArray[CurMolID].MolScaler->addChild(MolDArray[CurMolID].Display[realid]);
		}
		else
		{
			MolDArray[CurMolID].Display[realid]->removeChild(MolDArray[CurMolID].Display[realid]->getChild(0));
			AtomList->GetAtoms(MolDArray[CurMolID].MoleculeName);
			MolDArray[CurMolID].Display[realid]->addChild(MolDArray[CurMolID].GeoColor->GetColor(curcolor.c_str()));
		}
	}
	else
	{
		//printf("Not highlighting");
	}
}

void AtomView::dehighlightAll(){
	int numberOfMolecules = MolDArray.getSize();
	for (int i=0; i<numberOfMolecules; i++){
		for(int j=0; j<MolDArray[i].NumAtoms;j++)
		{
			//printf("i=%u, j=%u\n", i, j);
			MolDArray[i].Display[j]->removeChild(MolDArray[i].Display[j]->getChild(0));
			AtomList->GetAtoms(MolDArray[i].MoleculeName);
			string curcolor = MolDArray[i].findColorByName(AtomList->FileData->molecules[i].atom_types.getTypeName(New[j].type));
			//printf("%s,%s\n", AtomList->FileData->molecules[i].atom_types.getTypeName(New[j].type), curcolor.c_str());

			MolDArray[i].Display[j]->addChild(MolDArray[i].GeoColor->GetColor(curcolor.c_str()));
		}
	}
	for (int i=0; i<numberOfMolecules; i++){
		for(int j=0; j<MolDArray[i].NumBonds;j++)
		{
			//printf("i=%u, j=%u\n", i, j);
			MolDArray[i].BondDisplay[j]->removeChild(MolDArray[i].BondDisplay[j]->getChild(0));
			//AtomList->GetAtoms(MolDArray[i].MoleculeName);
			//string curcolor = MolDArray[i].findColorByName(AtomList->FileData->molecules[i].atom_types.getTypeName(New[j].type));
			//printf("%s,%s\n", AtomList->FileData->molecules[i].atom_types.getTypeName(New[j].type), curcolor.c_str());

			MolDArray[i].BondDisplay[j]->addChild(MolDArray[i].ColorData[7].ColorNode);
		}
	}
}
