#include <unistd.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <dpf.h>
#include <dtk.h>
#include <Performer/pf.h>
#include <Performer/pf/pfGeode.h>
#include <string>
#include <sstream>
#include "SysTimer.h"
using namespace std;
#include "AtomCommands.h"
#include "CommandParser.h"
#include "Messenger.h"
#include "FlexArrayTmpl.H"
#include "ReadAtomFiles.hh"
#include "SharedAtoms.h"
#include "colorhandler.h"
#include "atomview.h"


//Top level handler function that does all of the message handling and deletion
string AtomView::HandleMessages()
{
	string ACommand; //Temporary command holder
	//If we have messagers that are not any good then create them and hook them up
	//to ATOM_MESSAGE.  ATOM_MESSAGE is the shared memory filename that I (Andrew A. Ray)
	//have used for all of the message passing in this program
	if (message ==NULL)
		message = new CommandParser("ATOM_MESSAGE");
	if (SelfMessage == NULL)
		SelfMessage = new Messenger("ATOM_MESSAGE");

	//Get the message out of the message class by Mark J. Soltys II  (Modified by Andrew A. Ray to get around
	//   the use of environmental variables)
	message->getCommand(ACommand, "SNAP");
	//Call a subfunction to handle all of the actual message implementation
	DoCommand(ACommand);
	//If we are supposed to go away do so and clean up after ourselves
	if (Die)
	{
		//printf("Finished doing commands exiting\n");
		delete message;
		delete SelfMessage;
	}
	//For ease of use
	return ACommand;
}



//This actually implements all of the commands this program accepts
void AtomView::DoCommand(const string& ACommand)
{
	//Debug
	if (ACommand != "")
	{
		printf("Doing command %s\n", ACommand.c_str());
		//Exit the program
		if (!myTimer.isRunning()){
			myTimer.start();
			//printf("Not running, now starting\n");
		}
		else
		{
			myTimer.stop();
			double lapTime = myTimer.elapsedTime();
			printf("Time Elapsed=%f\n",lapTime);
			if ((lapTime) > 2)
			{
				printf("Dehighlighting");
				dehighlightAll();
			}
			myTimer.reset();
			myTimer.start();
		}
		if (ACommand == DIE)
		{
			Die=true;
			return;
		}
		if (ACommand == MOLECULE)
		{
			MoleculeName = message->getString();
			int moleculeid=-1;
			if (ReadStuff)
				moleculeid = AtomList->FileData->findMolecule(MoleculeName);
			if (moleculeid == -1){
				printf("Loading %s\n", MoleculeName.c_str());
				//printf("Got the file info command\n");
				MoleculeDisplay * newMol = new MoleculeDisplay(MoleculeName);
				//printf("Current order=%i\n",newMol->Order);
				//printf("NewMol Created\n");
				MolDArray.append(*newMol);
				if (AtomList == NULL){
					//printf("Making AtomList");
					AtomList = new DTKSharedAtoms("ATOM_MESSAGE", New, ArrayBondDefs, AtomsChanged);
					ReadStuff = true;
				}
				AtomList->WriteToSharedMem(MoleculeName);
				//printf("Write to Shared Memory done");
				CurMolID = AtomList->FileData->findMolecule(MoleculeName);
				//printf("CurrentMolID %u", CurMolID);
				setColors(MoleculeName);
				//printf("Set colors ok?\n");
				MakeTree();
				//printf("Tree made.\n");

				Load = true;
				//WorldScaleChange = true;
				//OriginChange=true;
			}
			else if(AtomList->FileData->molecules[moleculeid].Visible == false)
			{
				AtomList->FileData->molecules[moleculeid].Visible = true;
				MolDArray[moleculeid].Order = -1;
				printf("Loading %s\n", MoleculeName.c_str());
				//printf("Current moleculeid%u\n", moleculeid);
				float nodetrans = getCurrentTrans(moleculeid);
				//printf("Current Trans=%f\n", nodetrans);
				MolDArray[moleculeid].MolScaler->setTrans(nodetrans, 0, 0);
				Scaler->addChild(MolDArray[moleculeid].MolScaler);
				//AtomList->FileData->molecules[moleculeid].Visible == true;
				Load = true;
			}
			else if(AtomList->FileData->molecules[moleculeid].Visible == true)
			{
				printf("Unloading %s\n", MoleculeName.c_str());
				Scaler->removeChild(MolDArray[moleculeid].MolScaler);
				AtomList->FileData->molecules[moleculeid].Visible = false;
				MolDArray[moleculeid].Order = -2;
				int size = MolDArray.getSize();
				for (int i=0; i< size; i++)
				{
					if (MolDArray[i].Order != -2)
					{
						MolDArray[i].Order--;
					}
				}
				for (int i=0; i< size; i++)
				{
					if (MolDArray[i].Order != -2)
					{
						float nodetrans = getCurrentTrans(i);
						//printf("Nodetrans=%f, i=%u.\n", nodetrans, i);
						MolDArray[i].MolScaler->setTrans(nodetrans, 0, 0);
					}
				}

				Unload = true;

			}
			CurMolID = AtomList->FileData->findMolecule(MoleculeName.c_str());

			return;
		}

		if (ACommand == ATOM)
		{
			Atom = true;
			AtomName = message->getString();
			return;
		}

		if (ACommand == BOND)
		{
			Bond = true;
			BondName = message->getString();
			return;
		}

	}
}
